      SUBROUTINE DOP2D (CDON,CACC,C,DCDL,DCDV,PL,PV)
      INCLUDE 'TYPES.INC'

***   DOP2D - SETUP DOPING FROM TWO-DIMENSIONAL DATA
*
*     P.PICHLER      1985
*     S.SELBERHERR   1986
*
      PARAMETER (ERRLNK=0.1)
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION CDON(NXY),CACC(NXY),C(MAXDO2*MAXDO2),
     1DCDL(MAXDO2*MAXDO2),DCDV(MAXDO2*MAXDO2),PL(MAXDO2),PV(MAXDO2)

**    START OF EXECUTABLE CODE
*     ************************
*
*     SEARCH LINK BETWEEN SOURCE AND DRAIN PROFILE
      H=(X(NXG1)+X(NXG2))/2
      DO 1 ISS=NXG1+1,NXG2
      IF (X(ISS).GE.H) GOTO 2
 1    CONTINUE

 2    REWIND NUMDOP
      READ (NUMDOP,END=6) NPIECE

**    READ SOURCE PROFILE
*
*     DONOR CONCENTRATION
      CALL DOP2DR (C,DCDL,DCDV,PL,PV,1,1,NL,NV,NPIECE)
      DO 7701 L=1,NL
 7701 PL(L)=PL(L)+X(NXG1)
      IF (NPIECE.GT.0) THEN
         CALL BICUBI (PL,PV,C,DCDL,DCDV,X,Y(NYOX),CDON,NL,NV,
     1   NX,ISS-1,NY)
      ELSE
         CALL BICUB  (PL,PV,C,DCDL,DCDV,X,Y(NYOX),CDON,NL,NV,
     1   NX,ISS-1,NY,ONE,2)
      ENDIF

**    ACCEPTOR CONCENTRATION
      CALL DOP2DR (C,DCDL,DCDV,PL,PV,2,1,NL,NV,NPIECE)
      IF (NPIECE.GT.0) THEN
         CALL BICUBI (PL,PV,C,DCDL,DCDV,X,Y(NYOX),CACC,NL,NV,
     1   NX,ISS-1,NY)
      ELSE
         CALL BICUB  (PL,PV,C,DCDL,DCDV,X,Y(NYOX),CACC,NL,NV,
     1   NX,ISS-1,NY,ONE,2)
      ENDIF
      IF (ABS(NPIECE).EQ.1) THEN
         REWIND NUMDOP
         READ (NUMDOP) NPIECE
      ENDIF

**    READ DRAIN PROFILE
*
*     DONOR CONCENTRATION
      CALL DOP2DR (C,DCDL,DCDV,PL,PV,1,-1,NL,NV,NPIECE)
      DO 7702 L=1,NL
 7702 PL(L)=PL(L)+X(NXG2)
**    INTERPOLATION
      IF (NPIECE.GT.0) THEN
**       DERIVATIVES ON INPUT
         CALL BICUBI (PL,PV,C,DCDL,DCDV,X(ISS),Y(NYOX),CDON(ISS),NL,NV,
     1   NX,NX-ISS+1,NY)
      ELSE
**       NO DERIVATIVES ON INPUT
         CALL BICUB  (PL,PV,C,DCDL,DCDV,X(ISS),Y(NYOX),CDON(ISS),NL,NV,
     1   NX,NX-ISS+1,NY,ONE,2)
      ENDIF

**    ACCEPTOR CONCENTRATION
      CALL DOP2DR (C,DCDL,DCDV,PL,PV,2,-1,NL,NV,NPIECE)
**    INTERPOLATION
      IF (NPIECE.GT.0) THEN
**       DERIVATIVES ON INPUT
         CALL BICUBI (PL,PV,C,DCDL,DCDV,X(ISS),Y(NYOX),CACC(ISS),NL,NV,
     1   NX,NX-ISS+1,NY)
      ELSE
**       NO DERIVATIVES ON INPUT
         CALL BICUB  (PL,PV,C,DCDL,DCDV,X(ISS),Y(NYOX),CACC(ISS),NL,NV,
     1   NX,NX-ISS+1,NY,ONE,2)
      ENDIF

**    SCALE DONORS AND ACCEPTORS APPROPRIATELTY
      DO 3 L=1,NXY
      CDON(L)=CDON(L)/CNOR
 3    CACC(L)=CACC(L)/CNOR

**    SUBSTRATE DOPING
      SUBS=ABS(CDON(NXYMX+ISS)-CACC(NXYMX+ISS))

**    CHECK MATCHING AT LINK
      CTMIN=1.E14/CNOR
      JJ=ISS
      DO 4 J=1,NY
      CT1=CDON(JJ-1)+CACC(JJ-1)
      CT2=CDON(JJ)+CACC(JJ)
      IF (MIN(CT1,CT2).GT.CTMIN.AND.ABS(CT1-CT2).GT.ERRLNK*(CT1+CT2))
     1GOTO 5
 4    JJ=JJ+NX
      RETURN

**    DOPING PROFILE IS INCONSISTENT
 5    IFAIL=1
      RETURN

**    DOPING FILE EMPTY
 6    CALL ABTMOS (4)

      END
      SUBROUTINE DOP2DR (C,DCDL,DCDV,PL,PV,NPASS,INC,NL,NV,IDCDX)
      INCLUDE 'TYPES.INC'

***   DOP2DR - READ TWO-DIMENSIONAL DOPING PROFILE
*
*     P.PICHLER   1985
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION C(MAXDO2*MAXDO2),DCDL(MAXDO2*MAXDO2),
     1DCDV(MAXDO2*MAXDO2),PL(MAXDO2),PV(MAXDO2)

**    START OF EXECUTABLE CODE
*     ************************
*
      IF (NPASS.EQ.1) THEN
         READ (NUMDOP,END=4) NL,NV
         IF (NL.LT.10.OR.NL.GT.MAXDO2.OR.
     1       NV.LT.10.OR.NV.GT.MAXDO2) GOTO 5
         IF (INC.EQ.1) THEN
            JS=1
            JE=NL
         ELSE
            JS=NL
            JE=1
         ENDIF
         READ (NUMDOP,END=4) (PL(I),I=JS,JE,INC)
         IF (JS.NE.1) THEN
            DO 7701 L=1,NL
 7701       PL(L)=-PL(L)/XYNOR
         ELSE
            DO 7702 I=1,NL
 7702       PL(I)=PL(I)/XYNOR
         ENDIF
         READ (NUMDOP,END=4) (PV(I),I=1,NV)
         DO 7703 I=1,NV
 7703    PV(I)=PV(I)/XYNOR
      ELSE
         IF (INC.EQ.1) THEN
            JS=1
            JE=NL
         ELSE
            JS=NL
            JE=1
         ENDIF
      ENDIF
      JJ=0
      DO 1 I=1,NV
      READ (NUMDOP,END=4) (C(JJ+J),J=JS,JE,INC)
 1    JJ=JJ+NL
      IF (IDCDX.GT.0) THEN
         JJ=0
         DO 2 I=1,NV
         READ (NUMDOP,END=4) (DCDL(JJ+J),J=JS,JE,INC)
 2       JJ=JJ+NL
         JJ=0
         DO 3 I=1,NV
         READ (NUMDOP,END=4) (DCDV(JJ+J),J=JS,JE,INC)
 3       JJ=JJ+NL
         IF (JS.NE.1) THEN
            DO 7704 L=1,NL*NV
 7704       DCDL(L)=-DCDL(L)*XYNOR
         ELSE
            DO 7705 L=1,NL*NV
 7705       DCDL(L)=DCDL(L)*XYNOR
         ENDIF
         DO 7706 L=1,NL*NV
 7706    DCDV(L)=DCDV(L)*XYNOR
      ENDIF
      RETURN

**    INSUFFICIENT DATA ON DOPING FILE
 4    CALL ABTMOS (5)

**    DOPING FILE FORMAT ERROR
 5    CALL ABTMOS (6)

      END
