      SUBROUTINE BNDCON (D,B,E,A,C,F,NRANG)
      INCLUDE 'TYPES.INC'

***   BNDCON - SET BOUNDARY CONDITIONS IN COEFFICIENT MATRIX
*              FOR CONTINUITY EQUATIONS
*
*     S.SELBERHERR   1986
*
      INCLUDE 'CMOS02X.INC'
      DIMENSION D(NXT),B(NXT),E(NXT),A(NXT),C(NXT),F(NXT)

**    START OF EXECUTABLE CODE
*     ************************
*
*     SOURCE CONTACT
      DO 1 I=1,NXS
      D(I+NX)=0
      B(I+1)=0
      E(I)=-1
      A(I)=0
      C(I)=0
 1    F(I)=0

**    DRAIN CONTACT
      DO 2 I=NXD,NX
      D(I+NX)=0
      B(I)=0
      E(I)=-1
      A(I-1)=0
      C(I)=0
 2    F(I)=0

**    SUBSTRATE CONTACT
      IF (NT.EQ.NY.AND..NOT.LBOTI) THEN
*        DIRICHLET
         DO 3 I=NXYMX+1,NXY
 3       C(I-NX)=0
         NRANG=NY-1
      ELSE
*        NEUMANN
         NRANG=NT
      ENDIF

      END
      SUBROUTINE COECON (A,ICOEF)
      INCLUDE 'TYPES.INC'

***   COECON - COMPUTE REDUNDANT COEFFICIENTS FOR
*              CONTINUITY EQUATIONS
*
*     S.SELBERHERR   1986
*
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      DIMENSION A(NXYOX+NXT)

**    START OF EXECUTABLE CODE
*     ************************
*
*     DECIDE WHICH COEFFICIENT
      GOTO (1,2,3) ICOEF
      CALL ABTMOS(0)

**    SUB/SUP DIAGONAL
 1    CALL SLTGET (A,IPOI(1),NXYOX+NXT)
      DO 7701 L=1,NX-1
 7701 A(L)=DY(NYOX)/DX(L)
      DO 7702 L=NX,NXT
 7702 A(L)=A(L+NXYOX)
      IF (NT.NE.NY.OR.LBOTI) THEN
         DO 7703 L=1,NX-1
 7703    A(L+NXTMX)=DY(NYOX+NT-2)/DX(L)
      ENDIF
      GOTO 4

**    SUPER-SUB/SUPER-SUP DIAGONAL
 2    CALL SLTGET (A,IPOI(2),NXYOX+NXT)
      DO 7704 L=1,NXT
 7704 A(L)=A(L+NXYOX)
      IF (NT.NE.NY.OR.LBOTI) THEN
         DO 7705 L=1,NX
 7705    A(L+NXTMX)=0
      ENDIF
      GOTO 4

**    DISCRETIZATION AREAS
 3    CALL SLTGET (A,IPOI(4),NXYOX+NXT)
      DO 7706 L=1,NXT
 7706 A(L)=A(L+NXYOX)
      IF (NT.NE.NY) THEN
         DO 7707 L=1,NX
 7707    A(L+NXTMX)=DY(NYOX+NT-2)*DXM(L)/2
      ENDIF

4     RETURN
      END
      SUBROUTINE EQCON (PSI,A,B,C,D,E,F,G,CC1,KIND)
      INCLUDE 'TYPES.INC'

***   EQCON - SOLVE CONTINUITY EQUATION
*
*     S.SELBERHERR   1986-1988
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      DIMENSION PSI(NXY),A(NXY),B(NXY),C(NXY),D(NXY),E(NXY),F(NXY),
     1G(NXY),CC1(NXY),IS(2),IE(2)

**    START OF EXECUTABLE CODE
*     ************************
*
*     MINORITY/MAJORITY CARRIER CONCENTRATION
      CALL SLTGET (CC1,ICC(KIND),NXY)
**    IF NO EQUATION TO SOLVE
      IF (KIND.EQ.2.AND.IMODEL.LT.4) THEN
**       ASUME CONSTANT QUASI-FERMILEVEL FOR MAJORITIES
         H1=LOG(CINTR)+UB
         DO 1 J=1,NXYB
 1       F(J)=EXP(MIN(MAX(DPHMIN,H1-PSI(J)),DPHMAX))-CC1(J)
         CALL UPDCC (CC1,F,NXYB,.FALSE.,2)
      ELSE
**       REDUNDANT SUB/SUP DIAGONAL
         CALL COECON (A,1)
**       REDUNDANT SUPER-SUB/SUPER-SUP DIAGONAL
         CALL COECON (C,2)
**       PREPARE SUB/SUP DIAGONAL BY MULTIPLYING WITH DIFFUSIVITY
         CALL SLTGET (E,IDIF(KIND),NXT)
         DO 7701 L=1,NXT-1
 7701    A(L)=A(L)/2*(E(L)+E(L+1))
**       PREPARE SUPER-SUB/SUPER-SUP DIAGONAL
         IF (IDIF(KIND+2).NE.JSLPRE) CALL SLTGET (E,IDIF(KIND+2),NXT)
         DO 7702 L=1,NXTMX
 7702    C(L)=C(L)/2*(E(L)+E(L+NX))

**       SUB/SUP DIAGONAL
         CALL SLTGET (E,IBEP(KIND),NXT-1)
         CALL SLTGET (F,IBEB(KIND),NXT-1)
         B(1)=0
         DO 7703 L=1,NXT-1
         B(L+1)=A(L)*F(L)
 7703    A(L)=A(L)*E(L)

**       SUPER-SUB/SUPER-SUP DIAGONAL
         CALL SLTGET (E,IBEP(KIND+2),NXTMX)
         CALL SLTGET (F,IBEB(KIND+2),NXTMX)
         DO 7704 L=1,NX
 7704    D(L)=0
         DO 7705 L=1,NXTMX
         D(L+NX)=C(L)*F(L)
 7705    C(L)=C(L)*E(L)

**       IF RECOMBINATION
         IF (IMODEL.GE.4) THEN
**          MAJORITY/MINORITY CARRIER CONCENTRATION
            CALL SLTGET (E,ICC(3-KIND),NXT)
**          RECOMBINATION INTO F, DERIV. INTO E
            CALL RECOMB (F,E,CC1,KIND)
*           IF IMPACT IONIZATION
            IF (IGEN.NE.JSLPRE) THEN
**             GENERATION RATE INTO G
               CALL SLTGET (G,IGEN,NXT)
**             UPDATE RECOMBINATION
               DO 7706 L=1,NXT
 7706          F(L)=F(L)-G(L)
            ENDIF
**          DISCRETIZATION AREAS INTO G
            CALL COECON (G,3)
**          UPDATE RIGHT HAND SIDE
            F(1)=G(1)*F(1)
            E(1)=-G(1)*E(1)-B(2)
            G(1)=-B(2)
            DO 7707 L=2,NXT-1
            F(L)=G(L)*F(L)
            E(L)=-G(L)*E(L)-A(L-1)-B(L+1)
 7707       G(L)=-A(L-1)-B(L+1)
            F(NXT)=G(NXT)*F(NXT)
            E(NXT)=-G(NXT)*E(NXT)-A(NXT-1)
            G(NXT)=-A(NXT-1)
            DO 7708 L=1,NX
            E(L)=E(L)-D(L+NX)
 7708       F(L)=F(L)-(G(L)-D(L+NX))*CC1(L)
            DO 7709 L=NX+1,NXTMX
            E(L)=E(L)-C(L-NX)-D(L+NX)
 7709       F(L)=F(L)-(G(L)-C(L-NX)-D(L+NX))*CC1(L)
            DO 7710 L=NXTMX+1,NXT
            E(L)=E(L)-C(L-NX)
 7710       F(L)=F(L)-(G(L)-C(L-NX))*CC1(L)
         ELSE
            E(1)=-B(2)
            DO 7711 L=2,NXT-1
 7711       E(L)=-A(L-1)-B(L+1)
            E(NXT)=-A(NXT-1)
            DO 7712 L=1,NX
            E(L)=E(L)-D(L+NX)
 7712       F(L)=-E(L)*CC1(L)
            DO 7713 L=NX+1,NXTMX
            E(L)=E(L)-C(L-NX)-D(L+NX)
 7713       F(L)=-E(L)*CC1(L)
            DO 7714 L=NXTMX+1,NXT
            E(L)=E(L)-C(L-NX)
 7714       F(L)=-E(L)*CC1(L)
         ENDIF

**       RIGHT HAND SIDE FROM OPERATOR
         F(1)=F(1)-A(1)*CC1(2)
         DO 7715 L=2,NXT-1
 7715    F(L)=F(L)-A(L)*CC1(L+1)-B(L)*CC1(L-1)
         F(NXT)=F(NXT)-B(NXT)*CC1(NXT-1)
         DO 7716 L=1,NX
 7716    F(L)=F(L)-C(L)*CC1(L+NX)
         DO 7717 L=NX+1,NXTMX
 7717    F(L)=F(L)-C(L)*CC1(L+NX)-D(L)*CC1(L-NX)
         DO 7718 L=NXTMX+1,NXT
 7718    F(L)=F(L)-D(L)*CC1(L-NX)

**       SET BOUNDARY CONDITIONS
         CALL BNDCON (D,B,E,A,C,F,NRANG)

**       SOLVE LINEAR SYSTEM
         CALL SOLVNS (D,B,E,A,C,F,G,NX,NRANG)

**       COMPUTE INCREMENT NORM, DAMP EVENTUALLY AND UPDATE SOLUTION
         CALL UPDCC (CC1,F,NX*NRANG,.TRUE.,KIND)

**       CARE FOR DEEP BULK
         IF (LBOTI) THEN
            NYB=NY
         ELSE
            NYB=NY-1
         ENDIF
         IF (NRANG.LT.NYB) THEN
            JBOT=(NRANG-1)*NX
            DO 3 I=1,NX
            PHIBOT=PSI(I+JBOT)-LOG(CC1(I+JBOT))
            DO 3 J=NRANG+1,NYB
            JJ=(J-1)*NX
 3          CC1(I+JJ)=EXP(MIN(MAX(DPHMIN,PSI(I+JJ)-PHIBOT),DPHMAX))
         ENDIF

**       CURRENT DENSITIES
         CALL JCC (A,B,C,CC1,D,E,KIND)
      ENDIF

**    SAVE UPDATED CARRIER CONCENTRATION
      CALL SLTPUT (CC1,ICC(KIND),NXY)

      END
      SUBROUTINE GAUSS (D,B,A,C,F,P,Q,N,M)
      INCLUDE 'TYPES.INC'

***   GAUSS - SOLVE SYSTEM OF LINEAR EQUATIONS
*
*     A.SCHUETZ      1980-1983
*     S.SELBERHERR   1979-1987
*
      PARAMETER (DROP=1.E-8,NUMGAU=98)
      DIMENSION D(N*M),B(N*M),A(N*M),C(N*M),F(N*M),P(N*N),Q(2*N)

      NX=N
      NXX=NX*NX
      NY=M
      NX2=(NX-1)/2
      IADD=1-MOD(NX,2)
      NXH=NX2+IADD

**    OPEN EXTERNAL WORKING BUFFER
      OPEN (NUMGAU,ACCESS='DIRECT',FORM='UNFORMATTED',RECL=NXX*8)

**    INITIALIZE WORKING BUFFER
      DO 7701 L=1,NXX
 7701 P(L)=0

**    FIRST UNKNOWN (SECOND POINT)
*
      H=1/(1          -B(2)*A(1)-A(2)*B(3)-C(2)*D(2+NX))
      P(1)    =H*                A(2)*A(3)
      P(NXH)  =H*(     B(2)*C(1)          +C(2)*B(2+NX))
      P(NXH+1)=H*(               A(2)*C(3)+C(2)*A(2+NX))
      P(NX)   =H*                          C(2)*C(2+NX)
      F(2)    =H*(F(2)-B(2)*F(1)-A(2)*F(3)-C(2)*F(2+NX))

      IB=1
      DO 2 I=4,NX,2
      IB=IB+NX
      H=1/(1-B(I)*(A(I-1)+B(I-1)*P(IB-NX))-A(I)*B(I+1)-C(I)*D(I+NX))
      DO 7702 L=1,NX-1
 7702 P(L+IB-1)=H*B(I)*B(I-1)*P(L+IB-NX)
      P(IB)      =P(IB)      +H*A(I)*A(I+1)
      P(IB+NXH-1)=P(IB+NXH-1)+H*(B(I)*C(I-1)+C(I)*B(I+NX))
      P(IB+NXH)  =P(IB+NXH)  +H*(A(I)*C(I+1)+C(I)*A(I+NX))
      P(IB+NX-1) =            H*             C(I)*C(I+NX)
 2    F(I)=H*(F(I)-B(I)*(F(I-1)-B(I-1)*F(I-2))-A(I)*F(I+1)-C(I)*F(I+NX))

      JIP=0
      DO 5 J=2,NY
      JMOD2=MOD(J,2)
      NXH=NX2+IADD*JMOD2
      I=(J-1)*NX+JMOD2-1
      IF (JMOD2.EQ.1) THEN
         JIP=JIP+1
         WRITE (NUMGAU,REC=JIP) (P(L),L=1,NXX)
         IB=1-NX
      ENDIF

      DO 5 I1=JMOD2+1,NX,2
      I=I+2
      IB=IB+NX

**    FORWARD ELIMINATION OF RED POINTS
*
      H=D(I)*D(I-NX)
      IF (ABS(H).LE.DROP) THEN
         DO 7704 L=1,NX
 7704    Q(L)=0
      ELSE
         DO 7705 L=1,NX
 7705    Q(L)=H*P(L+IB-1)
      ENDIF
      Q(NXH)  =Q(NXH)  +D(I)*B(I-NX)+B(I)*D(I-1)
      Q(NXH+1)=Q(NXH+1)+D(I)*A(I-NX)
      Q(NX-1) =Q(NX-1)              +B(I)*B(I-1)
      Q(NX)   =Q(NX)   +D(I)*C(I-NX)+B(I)*A(I-1)-1
      IF (I.LT.NX*NY) THEN
         Q(NXH+1)=Q(NXH+1)+A(I)*D(I+1)
         Q(NX)   =Q(NX)   +A(I)*B(I+1)
         Q(NX+1) =         A(I)*A(I+1)
         IF (J.LT.NY) Q(NX)=Q(NX)+C(I)*D(I+NX)
      ELSE
         Q(NX+1)=0
      ENDIF
      DO 7706 L=2,NX
 7706 Q(L+NX)=0
      NN=NX+NXH
      Q(NN)=               B(I)*C(I-1)
      FI=F(I)-D(I)*F(I-NX)-B(I)*F(I-1)
      IF (I.LT.NX*NY) THEN
         Q(NN+1)=   A(I)*C(I+1)
         FI     =FI-A(I)*F(I+1)
         IF (J.LT.NY) THEN
            Q(NN)  =Q(NN)  +C(I)*B(I+NX)
            Q(NN+1)=Q(NN+1)+C(I)*A(I+NX)
            FI     =FI     -C(I)*F(I+NX)
         ENDIF
      ELSE
         Q(NN+1)=0
      ENDIF

**    FORWARD ELIMINATION OF BLACK POINTS
*
      IST=I-2*NX
      IF (IST.GT.0) THEN
         FI=FI+H*F(IST)
         IST=IST+2
         IB1=IB+NX-1
         JMIN=J-2
      ELSE
         IST=IST+NX-1
         INC=0
         IF (IST.EQ.0) INC=1
         IST=IST+2*INC
         IB1=(IST/2-1)*NX
         DO 7707 L=1,NX+2
 7707    Q(L)=Q(L+NXH+INC-1)
         Q(NN)=0
         Q(NN+1)=0
         JMIN=1
      ENDIF
      I3=0

***   START OF THE VERY EXPENSIVE PART
      DO 4 J1=JMIN,J
      IM=MIN(I-1,J1*NX)
      DO 3  I2=IST,IM,2
      I3=I3+1
      H=Q(I3)
      IF(ABS(H).GT.DROP) THEN
         DO 7708 L=1,NX
 7708    Q(I3+L)=Q(I3+L)+H*P(IB1+L)
         FI=FI+H*F(I2)
      ENDIF
      IB1=IB1+NX
 3    CONTINUE
      J1MOD2=MOD(J1,2)
      IF (J1MOD2.EQ.0) IB1=0
 4    IST=IM+2-J1MOD2
***   END OF THE VERY EXPENSIVE PART

      DO 7709 L=1,NX
 7709 Q(L)=Q(L+I3)
      IF (J.NE.NY) THEN
         Q(NX+1)=C(I)*C(I+NX)
      ELSE
         Q(NX+1)=0
      ENDIF
      H=-1/Q(1)
      F(I)=H*FI
      DO 7710 L=1,NX
 7710 P(L+IB-1)=H*Q(L+1)
 5    CONTINUE

**    BACKWARD SUBSTITUTION FOR BLACK POINTS
*
      DO 8 J=NY,1,-1
      JMOD2=MOD(J,2)
      IF (J.LT.NY.AND.JMOD2.EQ.0) THEN
         READ (NUMGAU,REC=JIP) (P(L),L=1,NXX)
         JIP=JIP-1
         IB=NXX-NX+1
      ENDIF
      I=J*NX-MOD(J+IADD,2)
      JMAX=MIN(J+2,NY)
      DO 8 I1=1+MOD(J+IADD,2),NX,2
      IB1=IB
      IST=I+2
      H=F(I)
      DO 7 J1=J,JMAX
      IM=MIN(I+2*NX,J1*NX)
      DO 6 I2=IST,IM,2
      H=H+F(I2)*P(IB1)
 6    IB1=IB1+1
 7    IST=IM+1+MOD(J1+1,2)

      F(I)=H
      I=I-2
 8    IB=IB-NX


**    BACKWARD SUBSTITUTION FOR RED POINTS
*
      F(1)=F(1)-A(1)*F(2)-C(1)*F(1+NX)
      DO 7712 L=3,NX,2
 7712 F(L)=F(L)-B(L)*F(L-1)-A(L)*F(L+1)-C(L)*F(L+NX)
      I1=NX+1
      I2=2*NX
      DO 9 J=2,NY-1
      L=I1+MOD(J-1,2)
      DO 7713 L=L,I2,2
 7713 F(L)=F(L)-D(L)*F(L-NX)-B(L)*F(L-1)-A(L)*F(L+1)-C(L)*F(L+NX)
      I1=I1+NX
 9    I2=I2+NX
      I1=I1+MOD(NY-1,2)
      DO 7714 L=I1,I2-1,2
 7714 F(L)=F(L)-D(L)*F(L-NX)-B(L)*F(L-1)-A(L)*F(L+1)
      IF (MOD(I1,2).EQ.MOD(I2,2)) F(I2)=F(I2)-D(I2)*F(I2-NX)-B(I2)*
     1F(I2-1)

**    CLOSE EXTERNAL WORKING BUFFER
      CLOSE (NUMGAU,STATUS='DELETE')

      END
      SUBROUTINE JCC (A,B,DIF,CC,CCJL,CCJT,KIND)
      INCLUDE 'TYPES.INC'

***   JCC - CALCULATE CURRENTDENSITY
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      DIMENSION A(NXT),B(NXT),DIF(NXT),CC(NXT),CCJL(NXT),CCJT(NXT)

**    START OF EXECUTABLE CODE
*     ************************
*
*     LATERAL CURRENT
      CALL SLTGET (A,IBEP(KIND),NXT-1)
      CALL SLTGET (B,IBEB(KIND),NXT-1)
      CALL SLTGET (DIF,IDIF(KIND),NXT)
      DO 7701 L=1,NXT-1
 7701 CCJL(L)=(DIF(L)+DIF(L+1))/2*(A(L)*CC(L+1)-B(L)*CC(L))
      DO 1 J=1,NT
      JJ=(J-1)*NX
      DO 1 L=JJ+1,JJ+NX-1
 1    CCJL(L)=CCJL(L)/DX(L-JJ)
      CALL SLTPUT (CCJL,IJD(KIND),NXT-1)

**    TRANSVERSAL CURRENT
      CALL SLTGET (A,IBEP(KIND+2),NXTMX)
      CALL SLTGET (B,IBEB(KIND+2),NXTMX)
      IF (IDIF(KIND+2).NE.JSLPRE) CALL SLTGET (DIF,IDIF(KIND+2),NXT)
      DO 7702 L=1,NXTMX
 7702 CCJT(L)=(DIF(L)+DIF(L+NX))/2*(A(L)*CC(L+NX)-B(L)*CC(L))
      DO 2 J=1,NT-1
      JJ=(J-1)*NX
      DO 2 L=JJ+1,JJ+NX
 2    CCJT(L)=CCJT(L)/DY(J+NYOX-1)
      CALL SLTPUT (CCJT,IJD(KIND+2),NXTMX)

**    TERMINAL CURRENTS
*
*     DRAIN CURRENT
      FLUXD(KIND)=0
      DO 3 J=1,NT-1
      JJ=(J-1)*NX+1
 3    FLUXD(KIND)=FLUXD(KIND)+DOTPRO(CCJT(JJ),FWX,NX)*DFWY(J)
      DO 4 J=1,NT
      JJ=(J-1)*NX+1
 4    FLUXD(KIND)=FLUXD(KIND)+DOTPRO(CCJL(JJ),DFWX,NX-1)*FWY(J)

**    BULK CURRENT
      IF (.NOT.LBOTI.AND.IMODEL.GE.4) THEN
         FLUXB(KIND)=0
         DO 5 J=1,NT-1
         JJ=(J-1)*NX+1
 5       FLUXB(KIND)=FLUXB(KIND)+DOTPRO(CCJT(JJ),DXM,NX)*DFWY(J)
         FLUXB(KIND)=FLUXB(KIND)/2
      ENDIF

      END
      SUBROUTINE SOLVNS (D,B,E,A,C,F,G,MX,MY)
      INCLUDE 'TYPES.INC'

***   SOLVNS - SOLVE NONSYMMETRIC LINEAR 5-POINT STAR SYSTEM
*
*     S.SELBERHERR   1986
*
      DIMENSION D(MX*MY),B(MX*MY),E(MX*MY),A(MX*MY),C(MX*MY),F(MX*MY),
*    1G(2*MIN(MX,MY))
     1G(*)

**    START OF EXECUTABLE CODE
*     ************************
*
      N=MX*MY
      FMAX=0
      DO 7701 L=1,N
      E(L)=1/E(L)
      F(L)=F(L)*E(L)
 7701 FMAX=MAX(ABS(F(L)),FMAX)
      FMAX=SQRT(FMAX)
      DO 7702 L=1,N
 7702 F(L)=F(L)/FMAX
      DO 7703 L=1,N-1
      A(L)=A(L)*E(L)
 7703 B(L+1)=B(L+1)*E(L+1)
      DO 7704 L=1,N-MX
      C(L)=C(L)*E(L)
 7704 D(L+MX)=D(L+MX)*E(L+MX)
      IF (MX.GT.MY) THEN
         CALL SWAP (A,E,MX,MY)
         CALL SWAP (C,A,MX,MY)
         CALL SWAP (B,C,MX,MY)
         CALL SWAP (D,B,MX,MY)
         CALL SWAP (F,D,MX,MY)
         CALL GAUSS (C,B,A,E,D,F,G,MY,MX)
         CALL SWAP (D,F,MY,MX)
      ELSE
         CALL GAUSS (D,B,A,C,F,E,G,MX,MY)
      ENDIF
      DO 7705 L=1,N
 7705 F(L)=F(L)*FMAX

      END
      SUBROUTINE SWAP (A,B,N,M)
      INCLUDE 'TYPES.INC'

***   SWAP - TRANSPOSE ARRAY
*
*     S.SELBERHERR   1979-1986
*
      DIMENSION A(N,M),B(M,N)

      DO 1 J=1,M
      DO 1 I=1,N
 1    B(J,I)=A(I,J)

      END
      SUBROUTINE UPDCC (CC,DCC,NXYA,LDAMP,KIND)
      INCLUDE 'TYPES.INC'

***   UPDCC - UPDATE CARRIER CONCENTRATION
*
*     S.SELBERHERR   1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION CC(NXYA),DCC(NXYA),IS(2),IE(2)

**    START OF EXECUTABLE CODE
*     ************************
*
      DO 1 L=1,NXS
 1    DCC(L)=0
      DO 2 L=NXD,NX
 2    DCC(L)=0
      IF (LDAMP) THEN
         DO 3 L=1,NXYA
         DC=DCC(L)
         DCC(L)=CC(L)
         IF (CC(L)+DC.LE.0) THEN
            IF (-DC*(1.1111*SMALL).LT.CC(L)) THEN
               DC=DC/(1-1.111*DC/CC(L))
            ELSE
               DC=0
            ENDIF
         ENDIF
 3       CC(L)=MIN(MAX(CCMIN,CC(L)+DC),CCMAX)
      ELSE
         DO 4 L=1,NXYA
         DC=DCC(L)
         DCC(L)=CC(L)
 4       CC(L)=MIN(MAX(CCMIN,CC(L)+DC),CCMAX)
      ENDIF
      PNORMD(KIND)=DIFNRM(CC,DCC,NXYA)

      END
