      SUBROUTINE AKID (Z,F,D,INC,N)
      INCLUDE 'TYPES.INC'

***   AKID - CALCULATE DERIVATIVES WITH AKIMA METHOD
*
*     P.PICHLER      1985
*     S.SELBERHERR   1986
*
      DIMENSION Z(N),F(INC*N),D(INC*N)

      ZR=Z(2)
      FR=F(INC+1)
      DR=(FR-F(1))/(ZR-Z(1))
      IF (N.NE.2) THEN
         ZRR=Z(3)
         FRR=F(2*INC+1)
         DRR=(FRR-FR)/(ZRR-ZR)
      ELSE
         ZRR=ZR
         FRR=FR
         DRR=DR
      ENDIF
      DL=2*DR-DRR
      H1=ABS(DRR-DR)
      H2=ABS(DR-DL)
      IF (H1+H2.EQ.0) THEN
         D(1)=(DL+DR)/2
      ELSE
         D(1)=(H1*DL+H2*DR)/(H1+H2)
      ENDIF
      DO 1 I=2,N
      J=(I-1)*INC+1
      ZR=ZRR
      FR=FRR
      DLL=DL
      DL=DR
      DR=DRR
      IF (I.LT.N-1) THEN
         ZRR=Z(I+2)
         FRR=F(J+2*INC)
         DRR=(FRR-FR)/(ZRR-ZR)
      ELSE
         DRR=2*DR-DL
      ENDIF
      H1=ABS(DRR-DR)
      H2=ABS(DL-DLL)
      IF (H1+H2.EQ.0) THEN
         D(J)=(DL+DR)/2
      ELSE
         D(J)=(H1*DL+H2*DR)/(H1+H2)
      ENDIF
 1    CONTINUE

**    MONOTONICITY CONSIDERATIONS
      DR=(F(INC+1)-F(1))/(Z(2)-Z(1))
      D(1)=SIGN(MIN(ABS(D(1)),3*ABS(DR)),D(1))
      DO 2 I=2,N-1
      J=(I-1)*INC+1
      DL=DR
      DR=(F(J+INC)-F(J))/(Z(I+1)-Z(I))
      D(J)=SIGN(MIN(ABS(D(J)),3*ABS(DL),3*ABS(DR)),D(J))
 2    CONTINUE
      J=J+INC
      DL=DR
      D(J)=SIGN(MIN(ABS(D(J)),3*ABS(DL)),D(J))

      END
      SUBROUTINE BICUB (XO,YO,UO,UXO,UYO,XN,YN,UN,NXO,NYO,NDXN,NXN,NYN,
     1UOFF,MODE)
      INCLUDE 'TYPES.INC'

***   BICUB - BICUBIC INTERPOLATTION BETWEEN MESHES WITHOUT
*             PARTIAL DERVIATIVES ON INPUT
*
*     S.SELBERHERR   1986
*
      DIMENSION XO(NXO),YO(NYO),UO(NXO*NYO),UXO(NXO*NYO),UYO(NXO*NYO),
     1XN(NXN),YN(NYN),UN(NDXN*NYN)

**    START OF EXECUTABLE CODE
*     ************************
*
      IF (MODE.EQ.1) THEN
**       INTERPOLATION OF LOGARITHMS REQUESTED
         DO 1 L=1,NXO*NYO
 1       UO(L)=LOG(UO(L))
      ELSEIF (MODE.EQ.2) THEN
**       INTERPOLATION OF LOGARITHMS WITH OFFSET REQUESTED
         DO 2 L=1,NXO*NYO
 2       UO(L)=LOG(UO(L)+UOFF)
      ENDIF

**    COMPUTE DERIATIVES
      L=1
      DO 3 J=1,NYO
      CALL AKID (XO,UO(L),UXO(L),1,NXO)
 3    L=L+NXO
      DO 4 L=1,NXO
 4    CALL AKID (YO,UO(L),UYO(L),NXO,NYO)

**    PERFORM INTERPOLATION
      CALL BICUBI (XO,YO,UO,UXO,UYO,XN,YN,UN,NXO,NYO,NDXN,NXN,NYN)

      IF (MODE.EQ.1) THEN
**       INTERPOLATION OF LOGARITHMS REQUESTED
         DO 5 J=1,NYN
         JJ=(J-1)*NDXN
         DO 5 L=1,NXN
 5       UN(JJ+L)=EXP(UN(JJ+L))
      ELSEIF (MODE.EQ.2) THEN
**       INTERPOLATION OF LOGARITHMS WITH OFFSET REQUESTED
         DO 6 J=1,NYN
         JJ=(J-1)*NDXN
         DO 6 L=1,NXN
 6       UN(JJ+L)=EXP(UN(JJ+L))-UOFF
      ENDIF

      END
      SUBROUTINE BICUBI (XO,YO,UO,UXO,UYO,XN,YN,UN,NXO,NYO,NDXN,NXN,NYN)
      INCLUDE 'TYPES.INC'

***   BICUBI - BICUBIC INTERPOLATTION BETWEEN MESHES WITH
*              PARTIAL DERVIATIVES ON INPUT
*
*     S.SELBERHERR   1986
*
      DIMENSION XO(NXO),YO(NYO),UO(NXO,NYO),UXO(NXO,NYO),UYO(NXO,NYO),
     1XN(NXN),YN(NYN),UN(NDXN,NYN)

**    START OF EXECUTABLE CODE
*     ************************
*
*     LOOP OVER ALL NEW GRID POINTS
      DO 5 J=1,NYN

**    SEARCH THE LOCATION OF YN(J) IN YO
      IF (YN(J).LE.YO(1)) THEN
**       NORTH OF NORTH BOUNDARY
         JJ=1
         YNH=0
      ELSEIF (YN(J).LT.YO(NYO)) THEN
**       WITHIN THE BOUNDARIES
         DO 1 JJ=NYO-1,1,-1
         IF (YN(J).GE.YO(JJ)) GOTO 2
 1       CONTINUE
 2       YNH=(YN(J)-YO(JJ))/(YO(JJ+1)-YO(JJ))
      ELSE
**       SOUTH OF SOUTH BOUNDARY
         JJ=NYO-1
         YNH=1
      ENDIF
      DYO=YO(JJ+1)-YO(JJ)

      DO 5 I=1,NXN

**    SEARCH THE LOCATION OF XN(I) IN XO
      IF (XN(I).LE.XO(1)) THEN
**       WEST OF THE WEST BOUNDARY
         II=1
         XNH=0
      ELSEIF (XN(I).LT.XO(NXO)) THEN
**       WITHIN THE BOUNDARIES
         DO 3 II=NXO-1,1,-1
         IF (XN(I).GE.XO(II)) GOTO 4
 3       CONTINUE
 4       XNH=(XN(I)-XO(II))/(XO(II+1)-XO(II))
      ELSE
**       EAST OF THE EAST BOUNDARY
         II=NXO-1
         XNH=1
      ENDIF

**    ABBREVIATIONS
      DXO=XO(II+1)-XO(II)

      UONW=UO(II,  JJ  )
      UONE=UO(II+1,JJ  )
      UOSW=UO(II  ,JJ+1)
      UOSE=UO(II+1,JJ+1)

      UXONW=UXO(II  ,JJ  )*DXO
      UXONE=UXO(II+1,JJ  )*DXO
      UXOSW=UXO(II  ,JJ+1)*DXO
      UXOSE=UXO(II+1,JJ+1)*DXO

      UYONW=UYO(II  ,JJ  )*DYO
      UYONE=UYO(II+1,JJ  )*DYO
      UYOSW=UYO(II  ,JJ+1)*DYO
      UYOSE=UYO(II+1,JJ+1)*DYO

      H=UOSW-UONW
      F3 = 3*H-2*UYONW-UYOSW
      F4 =-2*H+  UYONW+UYOSW
      H=UONW
**    THE IF-CONSTRUCT IS NOT EXECUTED IF (XNH.EQ.0) AND (YNH.EQ.0)
      IF (XNH.EQ.0) THEN
**       (0 .LE. YNH .LE. 1)
         IF (YNH.NE.0) THEN
**          (0 .LT. YNH .LE. 1)
            IF (YNH.NE.1) THEN
**             (0 .LT. YNH .LT. 1)
               H=H+YNH*(UYONW+YNH*(F3+YNH*F4))
            ELSE
**             (YNH .EQ. 1)
               H=H+UYONW+F3+F4
            ENDIF
         ENDIF
      ELSE
**       (0 .LT. XNH .LE. 1) AND (0 .LE. YNH .LE. 1)
         H1=UONE-H
         F1 = 3*H1-2*UXONW-UXONE
         F2 =-2*H1+  UXONW+UXONE
         H1=H1+UOSW-UOSE
         F7=UXONW-UXOSW
         F9=UYONW-UYONE
         F5=H1-F7-F9
         F6=-3*H1+2*F7+(UXONE-UXOSE)
         F7= 2*H1-  F7-(UXONE-UXOSE)
         F8=-3*H1+2*F9+(UYOSW-UYOSE)
         F9= 2*H1-  F9-(UYOSW-UYOSE)
         IF (XNH.NE.1) THEN
**          (0 .LT. XNH .LT. 1) AND (0 .LE. YNH .LE. 1)
            H=H+XNH*(UXONW+XNH*(F1+XNH*F2))
            IF (YNH.NE.0) THEN
**             (0 .LT. XNH .LT. 1) AND (0 .LT. YNH .LE. 1)
               H1=UYONW+XNH*(F5+XNH*(F6+XNH*F7))
               IF (YNH.NE.1) THEN
**                (0 .LT. XNH .LT. 1) AND (0 .LT. YNH .LT. 1)
                  H=H+YNH*(H1+YNH*(F3+YNH*F4+XNH*(F8+YNH*F9)))
               ELSE
**                (0 .LT. XNH .LT. 1) AND (YNH .EQ. 1)
                  H=H+H1+F3+F4+XNH*(F8+F9)
               ENDIF
            ENDIF
         ELSE
**          (XNH .EQ. 1) AND (0 .LE. YNH .LE. 1)
            H=H+UXONW+F1+F2
            IF (YNH.NE.0) THEN
**             (XNH .EQ. 1) AND (0 .LT. YNH .LE. 1)
               IF (YNH.NE.1) THEN
**                (XNH .EQ. 1) AND (0 .LT. YNH .LT. 1)
                  H=H+YNH*(UYONW+F5+F6+F7+YNH*(F3+F8+YNH*(F4+F9)))
               ELSE
**                (XNH .EQ. 1) AND (YNH .EQ. 1)
                  H=H+UYONW+F3+F4+F5+F6+F7+F8+F9
               ENDIF
            ENDIF
         ENDIF
      ENDIF
 5    UN(I,J)=H


      END
      SUBROUTINE BNDVAL (PSI,CMIN,CMAJ,CNET,LCCB)
      INCLUDE 'TYPES.INC'

***   BNDVAL - SET BOUNDARY VALUES
*
*     S.SELBERHERR   1979-1986
*

**    THE GEOMETRY IS AS FOLLOWS (*=DIRICHLET,-=NEUMAN BOUNDARY)
*                                (.=INTERFACE)
*
*        1 NXS NXG1      NXG2 NXD NX
*        !  !   !          !   !  !
*        **************************+1
*        ****   ************   ****
*        ****   ************   ****+NYG
*        ****                  ****
*        ****                  ****
*        ****..................****+NYOX
*        -                        -
*        -                        -
*        -                        -
*        -                        -
*        -                        -
*        -                        -
*        -                        -
*        -........................-+NYSEM   (DIRICHLET FOR BULK-MOS)
*        -                        -
*        -                        -
*        -                        -
*        --------------------------+NYTOT
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      DIMENSION PSI(NX,NYTOT),CMIN(NX,NY),CMAJ(NX,NY),CNET(NX,NY)

      PYT(REAL1,REAL2)=SQRT(REAL1**2+REAL2**2)

**    START OF EXECUTABLE CODE
*     ************************
*
      IF (LCCB) THEN
**       FIRST TREAT THE CARRIER CONCENTRATIONS
*
         H1=2*CINTR
         H2=CINTR**2
**       SOURCE CONTACT
         IS=1
         IE=NXS
         J=1
         DO 2 K=1,3
         DO 1 I=IS,IE
         H=PYT(CNET(I,J),H1)
         IF (CNET(I,J).LT.0) THEN
            CMAJ(I,J)=(H-CNET(I,J))/2
            CMIN(I,J)=MAX(H2/CMAJ(I,J),CCMIN)
         ELSE
            CMIN(I,J)=(H+CNET(I,J))/2
            CMAJ(I,J)=MAX(H2/CMIN(I,J),CCMIN)
         ENDIF
 1       CONTINUE
         IF (K.EQ.1) THEN
**          DRAIN CONTACT
            IS=NXD
            IE=NX
            J=1
         ELSEIF (K.EQ.2) THEN
**          BULK CONTACT
            IF (LBOTI) GOTO 3
            IS=1
            IE=NX
            J=NY
         ENDIF
 2       CONTINUE

**       SAVE CARRIER CONCENTRATIONS
 3       CALL SLTPUT (CMIN,ICC(1),NXY)
         CALL SLTPUT (CMAJ,ICC(2),NXY)
      ENDIF

**    TREAT THE ELECTROSTATIC POTENTIAL
*
*     GATE CONTACT
      DO 7701 J=1,NYG
      DO 7701 L=NXG1,NXG2
 7701 PSI(L,J)=UGS

**    SOURCE CONTACT
      H1=-LOG(CINTR)+UISN
      DO 7702 I=1,NXS
 7702 PSI(I,1)=H1+LOG(CMIN(I,1))
      DO 4 J=2,NYOX
      DO  4 L=1,NXS
 4    PSI(L,J)=PSI(L,1)

**    DRAIN CONTACT
      H1=-LOG(CINTR)+UD+UIDN
      DO 7703 I=NXD,NX
 7703 PSI(I,1)=H1+LOG(CMIN(I,1))
      DO 8 J=2,NYOX
      DO 8 L=NXD,NX
 8    PSI(L,J)=PSI(L,1)

**    SURFACE BETWEEN SOURCE AND GATE
      H=(PSI(NXG1,1)-PSI(NXS,1))/(X(NXG1)-X(NXS))
      DO 7704 L=NXS+1,NXG1-1
 7704 PSI(L,1)=PSI(NXS,1)+H*(X(L)-X(NXS))

**    SURFACE BETWEEN GATE AND DRAIN
      H=(PSI(NXD,1)-PSI(NXG2,1))/(X(NXD)-X(NXG2))
      DO 7705 L=NXG2+1,NXD-1
 7705 PSI(L,1)=PSI(NXG2,1)+H*(X(L)-X(NXG2))

**    BULK CONTACT
      IF (.NOT.LBOTI) THEN
         H1=LOG(CINTR)+UB
         DO 7706 L=1,NX
 7706    PSI(L,NYSEM)=H1-LOG(CMAJ(L,NY))
         IF (MODEL.GE.4.AND.RBULK.NE.0) THEN
**          ESTIMATE INFLUENCE OF BULK RESISTANCE
            DELPSI=MIN(PSI(1,NYOX)-4,UB-UF-(FLUXB(1)-FLUXB(2))*RBULK)-
     1      PSI(1,NYSEM)
            DO 10 I=1,NX
            RATIOB=0
            DO 9 J=NYOX+1,NYSEM
 9          RATIOB=RATIOB+DY(J-1)/(CINTR+CMAJ(I,J-NYOX+1))
            RATIOB=DELPSI/RATIOB
            RATIO=0
            DO 10 J=NYOX+1,NYSEM
            RATIO=RATIO+DY(J-1)/(CINTR+CMAJ(I,J-NYOX+1))
 10         PSI(I,J)=PSI(I,J)+RATIOB*RATIO
         ENDIF
      ENDIF

**    SAVE ELECTROSTATIC POTENTIAL
      CALL SLTPUT (PSI,IPSI,NXYTOT)

      END
      SUBROUTINE CHARGE (CRHO,CMIN,CMAJ,CDON,CACC,HLP,DCRHO,LDCRHO)
      INCLUDE 'TYPES.INC'

***   CHARGE - COMPUTE CHARGE CONCENTRATION
*
*     S.SELBERHERR   1986
*

**    A0=SQRT(PI)/2
      PARAMETER (A0=+1.772453850905516D+0/2)

      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'

      DIMENSION CRHO(NXY),CMIN(NXY),CMAJ(NXY),CDON(NXY),CACC(NXY),
     1HLP(NXY),DCRHO(NXY)

**    START OF EXECUTABLE CODE
*     ************************
*

      IF (LDCRHO) THEN
         IF (ITC(1).NE.JSLPRE.AND.ITC(2).NE.JSLPRE) THEN
            CALL SLTGET (HLP,ITC(1),NXY)
            CALL SLTGET (DCRHO,ITC(2),NXY)
            DO 7701 L=1,NXY
 7701       DCRHO(L)=CMIN(L)/HLP(L)+CMAJ(L)/DCRHO(L)
         ELSE
            DO 7702 L=1,NXY
 7702       DCRHO(L)=CMIN(L)+CMAJ(L)
         ENDIF
      ENDIF

      IF (ICHANL.GT.0) THEN
         IF (ECMED.GE.0) CALL IONIZE (CDON,DCRHO,CMIN,CSTAMI,ECMED,
     1   LDCRHO,NXY)
         IF (EAMEV.GE.0) CALL IONIZE (CACC,DCRHO,CMAJ,CSTAMA,EAMEV,
     1   LDCRHO,NXY)
         DO 7703 L=1,NXY
 7703    CRHO(L)=CDON(L)-(CMIN(L)-(CMAJ(L)-CACC(L)))
      ELSE
         IF (ECMED.GE.0) CALL IONIZE (CDON,DCRHO,CMAJ,CSTAMA,ECMED,
     1   LDCRHO,NXY)
         IF (EAMEV.GE.0) CALL IONIZE (CACC,DCRHO,CMIN,CSTAMI,EAMEV,
     1   LDCRHO,NXY)
         DO 7704 L=1,NXY
 7704    CRHO(L)=CACC(L)-(CMIN(L)-(CMAJ(L)-CDON(L)))
      ENDIF

      END
      SUBROUTINE DIFFX (DU,U,DZ,MX,MY)
      INCLUDE 'TYPES.INC'

***   DIFFX - DIFFERENTIATE WITH RESPECT TO X
*
*     S.SELBERHERR   1979-1986
*
      DIMENSION DU(MX*MY),U(MX*MY),DZ(*)

      DO 2 J=1,MY
      JJ=(J-1)*MX
      DO 1 L=JJ+1,JJ+MX-1
 1    DU(L)=(U(L+1)-U(L))/DZ(L-JJ)
 2    DU(JJ+MX)=0
      GOTO 5


      ENTRY DIFFY (DU,U,DZ,MX,MY)

***   DIFFY - DIFFERENTIATE WITH RESPECT TO Y
*
      DO 3 J=1,MY-1
      JJ=(J-1)*MX
      DO 3 L=JJ+1,JJ+MX
 3    DU(L)=(U(L+MX)-U(L))/DZ(J)
      DO 4 L=JJ+MX+1,MX*MY
 4    DU(L)=0

 5    RETURN
      END
      FUNCTION DIFNRM (X1,X2,N)
      INCLUDE 'TYPES.INC'

***   DIFNRM - COMPUTE NORM OF DIFFERENCE OF TWO VECTORS.
*
*     S.SELBERHERR   1979-1986
*
      PARAMETER (ACSLIM=0.9999)
      DIMENSION X1(N),X2(N)

      PYT(REAL1,REAL2)=SQRT(REAL1**2+REAL2**2)

      X1NRM=PYTV(X1,N)
      X2NRM=PYTV(X2,N)
      IF (X1NRM.NE.0) THEN
         IF (X2NRM.NE.0) THEN
            H1=1/X1NRM
            H2=1/X2NRM
            D12NRM=0
            DO 7701 L=1,N
 7701       D12NRM=D12NRM+(X1(L)*H1)*(X2(L)*H2)
            H1=LOG(X1NRM/X2NRM)
            H2=D12NRM
            IF (H2.GT.ACSLIM) THEN
               D12NRM=ABS(H1)
            ELSE
               H2=ACOS(H2)
               D12NRM=PYT(H1,H2)
            ENDIF
         ELSE
            D12NRM=1
         ENDIF
      ELSE
         IF (X2NRM.NE.0) THEN
            D12NRM=1
         ELSE
            D12NRM=0
         ENDIF
      ENDIF
      DIFNRM=D12NRM

      END
      FUNCTION DOTPRO (X1,X2,N)
      INCLUDE 'TYPES.INC'

***   DOTPRO - COMPUTE DOTPRODUCT OF TWO VECTORS.
*
*     S.SELBERHERR   1986
*
      DIMENSION X1(N),X2(N)

      H=0
      DO 1 L=1,N
 1    H=H+X1(L)*X2(L)
      DOTPRO=H

      END
      SUBROUTINE INTERX (U,UM,N,M,MODE)
      INCLUDE 'TYPES.INC'

***   INTERX - INTERPOLATE TO POINT VALUES (X-DIRECTION)
*
*     S.SELBERHERR   1979-1986
*
      DIMENSION U(N,M),UM(N,M)

      DO 1 J=1,M
      U(N,J)=UM(N-1,J)*MOD(MODE/2,2)
      DO 7701 L=N-1,2,-1
 7701 U(L,J)=(UM(L,J)+UM(L-1,J))/2
 1    U(1,J)=UM(1,J)*MOD(MODE,2)
      GOTO 3


      ENTRY INTERY (U,UM,N,M,MODE)

***   INTERY - INTERPOLATE TO POINT VALUES (Y-DIRECTION)
*
      DO 7702 L=1,N
 7702 U(L,M)=UM(L,M-1)*MOD(MODE/2,2)
      DO 2 J=M-1,2,-1
      DO 2 L=1,N
 2    U(L,J)=(UM(L,J)+UM(L,J-1))/2
      DO 7704 L=1,N
 7704 U(L,1)=UM(L,1)*MOD(MODE,2)

 3    RETURN
      END
      SUBROUTINE IONIZE (CDOP,DCRHO,CC,RNEFF,EACT,LDCRHO,N)
      INCLUDE 'TYPES.INC'

***   IONIZE - COMPUTE IONIZED DOPING CONCENTRATION
*
*     S.SELBERHERR   1987
*
      INCLUDE 'CMOS02X.INC'

      PARAMETER (CDLOW=1E18,CDHIGH=1E19)
      DIMENSION CDOP(N),DCRHO(N),CC(N)

      SAVE LINIT,CDLOWN,CDHIGN
      DATA LINIT /.FALSE./

**    START OF EXECUTABLE CODE
*     ************************
*
      IF (.NOT.LINIT) THEN
         CDLOWN=CDLOW/CNOR
         CDHIGN=CDHIGH/CNOR
         LINIT=.TRUE.
      ENDIF

      IF (LDCRHO) THEN
         DO 1 I=1,N
         H1=CC(I)/RNEFF
         IF (CDOP(I).LE.CDHIGN) THEN
            H1=H1*EACT
            IF (CDOP(I).LE.CDLOWN) THEN
               CDOP(I)=CDOP(I)/(1+H1)
               DCRHO(I)=DCRHO(I)+CDOP(I)*H1/(1+H1)
            ELSE
               H2=(CDOP(I)-CDLOWN)/(CDHIGN-CDLOWN)
               H3=(1-H2)/(1+H1)
               DCRHO(I)=DCRHO(I)+CDOP(I)*H3*H1/(1+H1)
               CDOP(I)=CDOP(I)*(H2+H3)
            ENDIF
         ENDIF
 1       CONTINUE
      ELSE
         DO 2 I=1,N
         H1=CC(I)/RNEFF
         IF (CDOP(I).LE.CDHIGN) THEN
            H1=H1*EACT
            IF (CDOP(I).LE.CDLOWN) THEN
               CDOP(I)=CDOP(I)/(1+H1)
            ELSE
               H2=(CDOP(I)-CDLOWN)/(CDHIGN-CDLOWN)
               H3=(1-H2)/(1+H1)
               CDOP(I)=CDOP(I)*(H2+H3)
            ENDIF
         ENDIF
 2       CONTINUE
      ENDIF

      END
      FUNCTION PYTV (X,N)
      INCLUDE 'TYPES.INC'

***   PYTV - COMPUTE EUCLIDEAN NORM
*
*     S.SELBERHERR   1979-1986
*
*     RELATIVE ERROR SMALLER THAN 20 DECIMAL DIGITS
*
*     FOR THE STUPID COMPILERS ONLY
      PARAMETER (FOUR=4)
      INCLUDE 'CMSYSX.INC'
      DIMENSION X(N)

*****************************************************
*     VERSION FOR COMPUTERS WITH LARGE EXPONENT RANGE
      P=0
      DO 1 I=1,N
 1    P=P+X(I)**2
      PYTV=SQRT(P)
*****************************************************
C*     VERSION FOR COMPUTERS WITH MEDIUM EXPONENT RANGE
C      P=0
C      SCAL=ABS(X(1))
C      DO 1 I=2,N
C 1    SCAL=MAX(SCAL,ABS(X(I)))
C      XMAX=1/SQRT(SMALL*N)
C      IF (SCAL.GE.XMAX) THEN
C         SCAL=XMAX/SCAL
C         DO 2 I=1,N
C 2       P=P+(X(I)*SCAL)**2
C         PYTV=SQRT(P)/SCAL
C      ELSE
C         DO 3 I=1,N
C 3       P=P+X(I)**2
C         PYTV=SQRT(P)
C      ENDIF
*****************************************************
C*     VERSION FOR COMPUTERS WITH SMALL EXPONENT RANGE
C      P=0
C      DO 1 I=1,N
C      Q=ABS(X(I))
C**    SETUP ITERATION PARAMETERS
C      IF (P.LT.Q) THEN
C         R=Q
C         Q=P
C         P=R
C      ENDIF
C
C      IF (P.NE.0) THEN
C**       1. ITERATION
C         R=(Q/P)**2
C         S=R/(FOUR+R)
C         IF (S.LE.EPSMA) GOTO 1
C         P=P+2*S*P
C         Q=S*Q
C**          2. ITERATION
C            R=(Q/P)**2
C            S=R/(FOUR+R)
C            IF (S.LE.EPSMA) GOTO 1
C            P=P+2*S*P
C            Q=S*Q
C**             3. ITERATION
C               R=(Q/P)**2
C               S=R/(FOUR+R)
C               IF (S.LE.EPSMA) GOTO 1
C               P=P+2*S*P
C               Q=S*Q
C**                4. ITERATION
C                  R=(Q/P)**2
C                  S=R/(FOUR+R)
C                  IF (S.LE.EPSMA) GOTO 1
C                  P=P+2*S*P
C      ENDIF
C 1    CONTINUE
C      PYTV=P
*****************************************************

      END
      SUBROUTINE QUAPHI (PSI,CC,PHI,KIND)
      INCLUDE 'TYPES.INC'

***   QUAPHI - CALCULATE QUASI-FERMILEVEL
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION PSI(NXT),CC(NXT),PHI(NXT)

**    START OF EXECUTABLE CODE
*     ************************
*
      H=LOG(CINTR)
      IF (KIND.EQ.1) THEN
**       MINORITY CARRIER QUASI-FERMILEVEL
         DO 7701 L=1,NXT
 7701    PHI(L)=PSI(L)-LOG(CC(L))+H
      ELSE
**       MAJORITY CARRIER QUASI-FERMILEVEL
         DO 7702 L=1,NXT
 7702    PHI(L)=PSI(L)+LOG(CC(L))-H
      ENDIF

      END
      SUBROUTINE SLTPUT (ARRAY,ISLT,N)
      INCLUDE 'TYPES.INC'

***   SLTPUT - PUT SLOT ONTO SCRATCH FILE
*     (NON VIRTUAL MEMORY VERSION)
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'

      PARAMETER (NUMSLT=99)
      DIMENSION ARRAY(N)
      SAVE NSLT

      IF (N.GT.0) THEN
         IF (ISLT.EQ.JSLPRE) THEN
            NSLT=NSLT+1
            ISLT=NSLT
         ENDIF
         WRITE (NUMSLT,REC=ISLT) (ARRAY(I),I=1,N)
      ENDIF
      RETURN


      ENTRY SLTGET (ARRAY,ISLT,N)

***   SLTGET - GET SLOT FROM SCRATCH FILE
*     (NON VIRTUAL MEMORY VERSION)
*
      IF (N.GT.0) READ (NUMSLT,REC=ISLT) (ARRAY(I),I=1,N)
      RETURN


      ENTRY SLTOPN (N)

***   SLTOPN - OPEN SCRATCH FILE WITH CORRECT SLOT SIZE
*     (NON VIRTUAL MEMORY VERSION)
*

**    ***********************************************************
*     * WATCH IN WHICH UNITS (BYTES,WORDS,ETC.) RECL HAS TO BE. *
*     * N IS HERE IN FLOATING POINT WORDS.                      *
*     ***********************************************************
*
      OPEN (NUMSLT,ACCESS='DIRECT',FORM='UNFORMATTED',RECL=N*8)
      NSLT=0
      RETURN


      ENTRY SLTCLS

***   SLTCLS - CLOSE SCRATCH FILE
*     (NON VIRTUAL MEMORY VERSION)
*
      CLOSE (NUMSLT,STATUS='DELETE')

      END
      SUBROUTINE SUB02
      INCLUDE 'TYPES.INC'

***   SUB02 - *MINIMOS* MATH.PROCESSOR
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      INCLUDE 'CPADBLK.INC'

      DATA IUB,IUD,IUG /3*0/

**    START OF EXECUTABLE CODE
*     ************************
*
*     ESTIMATE ALL INITIALLY REQUIRED DATA
      CALL SUB02A

**    PERFORM COMPUTATIONS
 1    UIDO=UIDN
      UISO=UISN
      UISN=(FLUXB(1)-FLUXB(2)+FLUXD(1)-FLUXD(2))*RSOURC
      UIDN=-(FLUXD(1)-FLUXD(2))*RDRAIN
      IF (IFAIL.EQ.0) CALL SUB02B
      IF (IFAIL.EQ.0.AND.MODEL.GE.3) THEN
         CALL SUB02C
         IF (IFAIL.EQ.0.AND.MODEL.GE.4) THEN
            CALL SUB02D
         ENDIF
      ENDIF
**    PERFORM SMALL SIGNAL ANALYSIS
C      IF (IFAIL.EQ.0.AND.LCAP) CALL SUB02E
**    PERFORM OUTPUT PROCESSING
      CALL SUB02F
      IF (IFAIL.EQ.1) GOTO 2
      IFAIL=0
      FLUXD(2)=0
      FLUXB(2)=0

**    DRAIN INCREMENT LOOP
      IUD=IUD+1
      UD=UD+DUD
      IF (IUD.LT.NUD) GOTO 1
      IUD=0
      UD=UD-NUD*DUD

**    GATE INCREMENT LOOP
      IUG=IUG+1
      UG=UG+DUG
      UGS=UGS+DUG
      IF (IUG.LT.NUG) GOTO 1
      IUG=0
      UG=UG-NUG*DUG
      UGS=UGS-NUG*DUG

**    BULK INCREMENT LOOP
      IUB=IUB+1
      UB=UB+DUB
      IF (IUB.LT.NUB) GOTO 1

**    CLOSE THE WORKING FILE
 2    CALL SLTCLS

      CALL ZZZSEC (HCPU)
      HCPU=HCPU-CPUTIM
      WRITE (NUMREF,'(/1X,F9.3,1X,A)') HCPU,'MINIMOS DONE'

      END
      SUBROUTINE SUB02I (LCLEAN,LREDUN,NKIND)
      INCLUDE 'TYPES.INC'

***   SUB02I - *ITERATION* PROCESSOR COMMON ROUTINE
*
*     S.SELBERHERR   1986
*
      PARAMETER (MAXCNV=1,MAXCYC=500)
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      INCLUDE 'CPADBLK.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
*     WRITE REFERENCE FILE HEADER
      IF (ITCYC.EQ.0) THEN
         IF (.NOT.LBOTI.AND.IMODEL.GE.4) THEN
            WRITE (NUMREF,'(/''       CPU   IT  L-(MAJ)'',
     1     ''  L-(MIN)   L-(F)   L-(PSI) NIT    DRAIN     BULK'')')
         ELSE
            WRITE (NUMREF,'(/''       CPU   IT  L-(MAJ)'',
     1     ''  L-(MIN)   L-(F)   L-(PSI) NIT    DRAIN'')')
         ENDIF
      ENDIF

      IF (LCLEAN) THEN
**       CLEAN UP WORKING FILE
         CALL SLTGET (PAD(JP(1)),IPSI  ,NXYTOT)
         CALL SLTGET (PAD(JP(2)),ICC(1),NXY)
         CALL SLTGET (PAD(JP(3)),ICC(2),NXY)
         CALL SLTGET (PAD(JP(4)),ICDON ,NXY)
         CALL SLTGET (PAD(JP(5)),ICACC ,NXY)
         IF (.NOT.LREDUN) THEN
            CALL SLTGET (PAD(JP(6)),IPOI(1),NXYTOT)
            CALL SLTGET (PAD(JP(7)),IPOI(2),NXYTOT)
            CALL SLTGET (PAD(JP(8)),IPOI(3),NXYTOT)
            CALL SLTGET (PAD(JP(9)),IPOI(4),NXYOX+NXY)
         ENDIF
         CALL SLTCLS
         DO 1 I=1,MAXSLT
 1       IMEM(I)=JSLPRE
         CALL SLTOPN (NXYTOT)
         CALL SLTPUT (PAD(JP(1)),IPSI  ,NXYTOT)
         CALL SLTPUT (PAD(JP(2)),ICC(1),NXY)
         CALL SLTPUT (PAD(JP(3)),ICC(2),NXY)
         CALL SLTPUT (PAD(JP(4)),ICDON ,NXY)
         CALL SLTPUT (PAD(JP(5)),ICACC ,NXY)
         IF (.NOT.LREDUN) THEN
            CALL SLTPUT (PAD(JP(6)),IPOI(1),NXYTOT)
            CALL SLTPUT (PAD(JP(7)),IPOI(2),NXYTOT)
            CALL SLTPUT (PAD(JP(8)),IPOI(3),NXYTOT)
            CALL SLTPUT (PAD(JP(9)),IPOI(4),NXYOX+NXY)
         ENDIF
      ENDIF

      IF (LREDUN) THEN
**       CALCULATE REDUNDANT POISSON COEFFICIENTS
         CALL COEPOI (PAD(JP(6)),PAD(JP(7)),PAD(JP(8)),PAD(JP(9)))
**       CALCULATE TERMINAL CURRENT INTEGRATION WEIGHTS
         CALL WNRM
      ENDIF

**    RESET ITERATION PARAMETERS
      PNORM=0
      IPM=0
      IPMPT=0
      ITCYCC=ITCYC
      ITCYCO=ITCYC
      LAVLUP=.FALSE.
      LFRST=.TRUE.

**    START OF OVER-ALL ITERATION LOOP
 2    ITCYC=ITCYC+1
**    CALCULATE BERNOULLI WEIGHTS
 3    CALL BERCO (PAD(JP(1)+NXYOX),PAD(JP(2)),PAD(JP(3)),PAD(JP(4)),
     1PAD(JP(5)),PAD(JP(6)),PAD(JP(7)),NKIND)
**    COMPUTE DIFFUSIVITY DISTRIBUTION
      CALL MOBIL (PAD(JP(1)+NXYOX),PAD(JP(2)),PAD(JP(3)),PAD(JP(4)),
     1PAD(JP(5)),PAD(JP(6)),PAD(JP(7)),PAD(JP(8)),PAD(JP(9)),
     2PAD(JP(10)),NKIND)

**    CHECK FOR INNER CARRIER TEMPERATURE ITERATION
      IF (IMODEL.NE.5) GOTO 4
**    IF HOT UPDATED FULFILLED
      IF (PNORH.LE.ERRTC) GOTO 7
**    RESET PRESENT MODEL
      IMODEL=4
      IPM=0
      IPMPT=0
      PNORH=2*ERRTC

 4    CONTINUE
**    SOLVE CONTINUITY EQUATION FOR MAJORITIES
      CALL EQCON (PAD(JP(1)+NXYOX),PAD(JP(2)),PAD(JP(3)),PAD(JP(4)),
     1PAD(JP(5)),PAD(JP(6)),PAD(JP(7)),PAD(JP(8)),PAD(JP(9)),2)
**    SOLVE CONTINUITY EQUATION FOR MINORITIES
      CALL EQCON (PAD(JP(1)+NXYOX),PAD(JP(2)),PAD(JP(3)),PAD(JP(4)),
     1PAD(JP(5)),PAD(JP(6)),PAD(JP(7)),PAD(JP(8)),PAD(JP(9)),1)
**    SOLVE POISSON EQUATION
      CALL EQPOI (PAD(JP(1)),PAD(JP(2)),PAD(JP(3)),PAD(JP(4)),
     1PAD(JP(5)),PAD(JP(6)),PAD(JP(7)),PAD(JP(8)),PAD(JP(9)),ITLIN,
     1LFRST)
      IF (IPM.GT.1) THEN
         ITCYCA=-ITCYC
      ELSE
         ITCYCA=ITCYC
      ENDIF
      IF (ITCYC.NE.ITCYCO) THEN
         CALL ZZZSEC (HCPU)
         HCPU=HCPU-CPUTIM
         IF (.NOT.LBOTI.AND.IMODEL.GE.4) THEN
            WRITE (NUMREF,
     1      '(1X,F9.3,I5,1P4E9.1,I4,1P2E10.2)')
     2      HCPU,ITCYCA,
     3      PNORMD(2),PNORMD(1),PNORM,PNORMD(3),ITLIN,
     4      (FLUXD(1)-FLUXD(2))*FLUNOR,(FLUXB(1)-FLUXB(2))*FLUNOR
         ELSE
            WRITE (NUMREF,
     1      '(1X,F9.3,I5,1P4E9.1,I4,1PE10.2)')
     2      HCPU,ITCYCA,
     3      PNORMD(2),PNORMD(1),PNORM,PNORMD(3),ITLIN,
     4      (FLUXD(1)-FLUXD(2))*FLUNOR
         ENDIF
         ITCYCO=ITCYC
      ENDIF

**    IF WE TRIED TOO OFTEN
      IF (ITCYC.EQ.MAXCYC) GOTO 5
**    IF WE ARE NOT SUFFICIENTLY ACCURATE
      H=0.1*MAX(ABS(UIDN-UIDO),ABS(UISN-UISO))*UT
      IF (MAX(PNORM,H,PNORMD(1),PNORMD(2)).GT.ERROR.OR.
     1    PNORMD(3).GT.EPSMA) GOTO 2
      IF (ITCYC.LE.ITCYCC+2.AND.IGEN.NE.JSLPRE) THEN
         ICNV=ICNV+1
      ELSE
         ICNV=0
      ENDIF
      ITCYCC=ITCYC
      IF ((ICNV.GE.MAXCNV.AND.LAVLUP).OR.IMODEL.LT.4) GOTO 7

**    CHECK FOR EXPLODING CURRENT
      IF (2*ABS(FLUX2D).LT.ABS(FLUXD(1)-FLUXD(2)).AND.
     1ABS(FLUX2D*FLUNOR).GT.1.E-6.AND.LPHYCK) GOTO 6

**    IMPACT IONIZATION RATE
      CALL GENRAT (PAD(JP(1)+NXYOX),PAD(JP(5)),PAD(JP(6)),PAD(JP(7)),
     1PAD(JP(8)),PAD(JP(9)))
      LAVLUP=.TRUE.
      IPM=0
      IPMPT=0
      LFRST=.TRUE.
      GOTO 2

 5    IFAIL=2
      GOTO 7

 6    IFAIL=4
 7    RETURN
      END
