      SUBROUTINE BWFDOP (CNET,BWFX,BWFY,HLP)
      INCLUDE 'TYPES.INC'

***   BWFDOP - COMPUTE WEIGHT FUNCTION FROM DOPING PROFILE
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION CNET(NX,NY),BWFX(MAXNPT-1),BWFY(MAXNPT-1),HLP(NX,NY)

**    START OF EXECUTABLE CODE
*     ************************
*
*     MINIMUM SPACING IS 200 ANGSTROM
      XYMIN=20E-7*SECAUR/XYNOR
**    TRANSFORM DOPING PROFILE SUITABLY
      DOPREF=SUBS
      DO 1 J=1,NY
      DO 1 I=1,NX
 1    HLP(I,J)=0.35*SIGN(LOG10(MAX(ONE+1E-3,ABS(CNET(I,J))/DOPREF))
     1**1.25,CNET(I,J))

**    COMPUTE WEIGHT FUNCTION OF X-MESH
      DO 3 I=1,NX-1
      IF (DX(I).LE.XYMIN) THEN
         BWFX(I)=0
      ELSE
         H=0
         DO 2 J=1,NY
 2       H=MAX(H,ABS(HLP(I,J)-HLP(I+1,J)))
         BWFX(I)=H
      ENDIF
 3    CONTINUE

**    COMPUTE WEIGHT FUNCTION OF Y-MESH
*
*     ASSUME OXIDE MESH TO BE ALREADY APPROPRIATE
      DO 7701 L=1,NYOX-1
 7701 BWFY(L)=0
      DO 5 J=1,NY-1
      IF (DY(J+NYOX-1).LE.XYMIN) THEN
         BWFY(J+NYOX-1)=0
      ELSE
         H=0
         DO 4 I=1,NX
 4       H=MAX(H,ABS(HLP(I,J)-HLP(I,J+1)))
         BWFY(J+NYOX-1)=H
      ENDIF
 5    CONTINUE

**    BE CRITICAL WITH EXTENSION INTO BULK
      IF (.NOT.LBOTI) BWFY(NY+NYOX-2)=2*BWFY(NY+NYOX-2)

      DO 6 J=NYSEM,NYTOT-1
 6    BWFY(J)=0

      END
      SUBROUTINE GAPPAR (T,EGAP,RMNEFF,RMPEFF,RNC,RNV,UT,CINTRI)
      INCLUDE 'TYPES.INC'

***   GAPPAR - COMPUTE GAP PARAMETERS
*
*     S.SELBERHERR   1986
*
      RMNEFF=(1.045+T*4.5E-4)
      RMPEFF=(0.523+T*(1.4E-3-T*1.48E-6))
      IF (T.GT.170.) THEN
         EGAP=1.1785-T*(9.025E-5+T*3.05E-7)
      ELSE
         EGAP=1.17+T*(1.059E-5-T*6.05E-7)
      ENDIF
      H1=EXP(45.13+0.75*LOG(RMNEFF*RMPEFF*(T/300)**2))
      H2=SQRT(SQRT((RMNEFF/RMPEFF)**3))
      RNC=H1*H2
      RNV=H1/H2
      UT=1.380662E-23*T/1.6021892E-19
      CINTRI=H1*EXP(-EGAP/(2*UT))

      END
      SUBROUTINE GENRAT (PSI,EL,ET,CJL,CJT,GEN)
      INCLUDE 'TYPES.INC'

***   GENRAT - ESTIMATE GENERATION RATE
*
*     A.SCHUETZ      1980-1983
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      DIMENSION PSI(NXY),EL(NXY),ET(NXY),CJL(NXY),CJT(NXY),GEN(NXY)

      PYT(REAL1,REAL2)=SQRT(REAL1**2+REAL2**2)

**    START OF EXECUTABLE CODE
*     ************************
*
*     ELECTRICAL FIELD AT MESH POINTS
      CALL DIFFX (GEN,PSI,DX,NX,NY)
      CALL INTERX (EL,GEN,NX,NY,0)
      CALL DIFFY (GEN,PSI,DY(NYOX),NX,NY)
      CALL INTERY (ET,GEN,NX,NY,3)

**    MINORITY CARRIER CONTRIBUTION
*
*     CURRENT DENSITY AT MESHPOINTS
      CALL SLTGET (CJT,IJD(1),NXY-1)
      CALL INTERX (CJL,CJT,NX,NY,0)
      CALL SLTGET (CJT,IJD(3),NXYMX)
      IF (LBOTI) THEN
         MODI=0
      ELSE
         MODI=2
      ENDIF
      CALL INTERY (CJT,CJT,NX,NY,MODI)

**    CALCULATE THE AVALANCHE RATE
*     G = / J / * ALPHA * EXP( BETA / EPARALLEL )
      ALPHAI=ALPHA(1)
      BETAI=BETA(1)
      ESM=-BETAI*0.05
      DO 1 L=1,NXY
      CUR=PYT(CJL(L),CJT(L))
      GEN(L)=0
      IF (CUR.GT.SMALL) THEN
         EPAR=-EL(L)*(CJL(L)/CUR)-ET(L)*(CJT(L)/CUR)
         IF (EPAR.GE.ESM) GEN(L)=CUR*ALPHAI*EXP(BETAI/EPAR)
      ENDIF
 1    CONTINUE

**    MAJORITY CARRIER CONTRIBUTION
*
*     CURRENT DENSITY AT MESHPOINTS
      CALL SLTGET (CJT,IJD(2),NXY-1)
      CALL INTERX (CJL,CJT,NX,NY,0)
      CALL SLTGET (CJT,IJD(4),NXYMX)
      CALL INTERY (CJT,CJT,NX,NY,MODI)

**    CALCULATE THE AVALANCHE RATE
*     G = / J / * ALPHA * EXP( BETA / EPARALLEL )
      ALPHAI=ALPHA(2)
      BETAI=BETA(2)
      ESM=-BETAI*0.03
      DO 2 L=1,NXY
      CUR=PYT(CJL(L),CJT(L))
      IF (CUR.GT.SMALL) THEN
         EPAR=EL(L)*(CJL(L)/CUR)+ET(L)*(CJT(L)/CUR)
         IF (EPAR.GE.ESM) GEN(L)=GEN(L)+CUR*ALPHAI*EXP(BETAI/EPAR)
      ENDIF
 2    CONTINUE

**    SAVE GENERATION RATE
      CALL SLTPUT (GEN,IGEN,NXY)

      END
      SUBROUTINE GRID
      INCLUDE 'TYPES.INC'

***   GRID - BUILD VERY INITIAL GRID
*
*     S.SELBERHERR   1979-1986
*     W.MADER        1987        SIEMENS VIENNA
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
*     MINIMUM SPACING IS 10 ANGSTROM
      XYMIN=1E-7*SECAUR/XYNOR

**    BUILD MESH IN X-DIRECTION
      X(1)=-SECAUR*(SGAP+XOFF)
      X(2)=-(SGAP+XOFF)
      X(3)=0
      X(4)=XLGEO
      X(5)=XLGEO+(XOFF+DGAP)
      X(6)=XLGEO+SECAUR*(XOFF+DGAP)
      NX=6

      K=6
      XINC=XLGEO/K
**    HANDLE INITIAL GRID FOR SURFACE STATES
      IF (XISS.GT.-1) THEN
         DS1=MAX(XYMIN,MIN(SISS,(XOFF+SGAP)/4))
         DS2=MAX(XYMIN,MIN(SISS,XINC))
         DS3=MAX(XYMIN,MIN(SISS,(XOFF+DGAP)/4))

         IF (X(3)+1.5*DS2.LE.XISS.AND.XISS.LE.X(4)-1.5*DS2) THEN
**          UNDER GATE
            XISSN=MIN((X(4)-2*DS2),MAX(XISS,(X(3)+2*DS2)))
            CALL GRIDS (3,4,NX,MAXNPT,X)
            X(6)=XISSN+DS2
            X(5)=XISSN
            X(4)=XISSN-DS2
            CALL GRIDF (3,7,K,SECAUR,NX,MAXNPT,X)
         ELSEIF (SISS.GT.DS1*1.5.AND.
     1           X(2)+DS1.LE.XISS.AND.XISS.LE.X(3)-DS1) THEN
**          BETWEEN SOURCE AND GATE
            CALL GRIDS (1,3,NX,MAXNPT,X)
            X(3)=(X(2)+X(3))/2
            CALL GRIDF (4,5,K,SECAUR,NX,MAXNPT,X)
         ELSEIF (SISS.GT.DS2*1.5.AND.
     1          X(4)+DS3.LE.XISS.AND.XISS.LE.X(5)-DS3) THEN
**          BETWEEN GATE AND DRAIN
            CALL GRIDS (1,5,NX,MAXNPT,X)
            X(5)=(X(4)+X(5))/2
            CALL GRIDF (3,4,K,SECAUR,NX,MAXNPT,X)
         ELSEIF (X(2)+1.5*DS1.LE.XISS.AND.XISS.LE.X(3)-1.5*DS1) THEN
**          BETWEEN SOURCE AND GATE
            XISSN=MIN((X(3)-2*DS1),MAX(XISS,(X(2)+2*DS1)))
            CALL GRIDS (3,3,NX,MAXNPT,X)
            X(5)=XISSN+DS1
            X(4)=XISSN
            X(3)=XISSN-DS1
            CALL GRIDF (6,7,K,SECAUR,NX,MAXNPT,X)
         ELSEIF (X(4)+1.5*DS3.LE.XISS.AND.XISS.LE.X(5)-1.5*DS3) THEN
**          BETWEEN GATE AND DRAIN
            XISSN=MIN((X(5)-2*DS3),MAX(XISS,(X(4)+2*DS3)))
            CALL GRIDS (3,5,NX,MAXNPT,X)
            X(7)=XISSN+DS3
            X(6)=XISSN
            X(5)=XISSN-DS3
            CALL GRIDF (3,4,K,SECAUR,NX,MAXNPT,X)
         ELSEIF (X(3)-1.5*DS1.LT.XISS.AND.XISS.LT.X(3)+1.5*DS2) THEN
**          LEFT GATE EDGE
            CALL GRIDS (2,4,NX,MAXNPT,X)
            IF(XISS.LT.X(3)-DS1/2) THEN
               IFIX=5
               X(5)=X(3)
               X(4)=X(5)-DS1
               X(3)=X(4)-DS1
            ELSEIF (XISS.GT.X(3)+DS2/2) THEN
               IFIX=3
               X(4)=X(3)+DS2
               X(5)=X(4)+DS2
            ELSE
               IFIX=4
               X(4)=X(3)
               X(3)=X(4)-DS1
               X(5)=X(4)+DS2
            ENDIF
            CALL GRIDF (IFIX,6,K,SECAUR,NX,MAXNPT,X)
         ELSEIF (X(4)-1.5*DS2.LT.XISS.AND.XISS.LT.X(4)+1.5*DS3) THEN
**          RIGHT GATE EDGE
            CALL GRIDS (2,5,NX,MAXNPT,X)
            IF(XISS.LT.X(4)-DS2/2) THEN
               IFIX=6
               X(6)=X(4)
               X(5)=X(6)-DS2
               X(4)=X(5)-DS2
            ELSEIF (XISS.GT.X(4)+DS3/2) THEN
               IFIX=4
               X(5)=X(4)+DS3
               X(6)=X(5)+DS3
            ELSE
               IFIX=5
               X(5)=X(4)
               X(4)=X(5)-DS2
               X(6)=X(5)+DS3
            ENDIF
            CALL GRIDF (3,IFIX,K,SECAUR,NX,MAXNPT,X)
         ENDIF
      ELSE
         CALL GRIDF (3,4,K,SECAUR,NX,MAXNPT,X)
      ENDIF
**    COMPUTE INCREMENTS
      DO 1 I=1,NX-1
 1    DX(I)=X(I+1)-X(I)
**    CHECK MESH FOR QUASI-UNIFORMITY
      CALL GRIDC (X,DX,XYMIN,NX,MAXNPT,SECAUR)

**    BUILD MESH IN Y-DIRECTION
      H=TINS
      Y(1)=-2*H
      Y(2)=-H
      J=3
 2    H=H/SECAUR
      Y(J)=-H
      J=J+1
      IF (H.GT.XYMIN) GOTO 2
      DYSAVE=H
      Y(J)=0
      J=J+1
      Y(J)=H/SECAUR
      IF (.NOT.LBOTI) THEN
 3       J=J+1
         Y(J)=Y(J-1)+H
         H=H*SECAUR
**       AT LEAST ONE MICRON THICK SUBSTRATE
         IF (Y(J).LT.1E-4/XYNOR) GOTO 3
      ELSE
 4       J=J+1
         Y(J)=Y(J-1)+H
         H=H*SECAUR
         IF (Y(J).LT.YSEMI) GOTO 4
         H=DYSAVE*SECAUR
         Y(J-1)=YSEMI-H
         Y(J)=YSEMI
 5       J=J+1
         Y(J)=Y(J-1)+H
         H=H*SECAUR
         IF (Y(J).LT.YSEMI+YISUBS) GOTO 5
         J=J-1
         Y(J)=YSEMI+YISUBS
      ENDIF
      NYTOT=J
**    COMPUTE INCREMENTS
      DO 6 J=1,NYTOT-1
 6    DY(J)=Y(J+1)-Y(J)
**    CHECK MESH FOR QUASI-UNIFORMITY
      CALL GRIDC (Y,DY,XYMIN,NYTOT,MAXNPT,SECAUR)

      END
      SUBROUTINE GRIDC (Z,DZ,DZMIN,NZ,NZMAX,SECAUR)
      INCLUDE 'TYPES.INC'

***   GRIDC - CHECK VERY INITIAL GRID FOR UNIFORMITY
*
*     S.SELBERHERR  1986
*
      DIMENSION Z(NZMAX),DZ(NZMAX-1)

**    START OF EXECUTABLE CODE
*     ************************
*
*     CHECK TO THE RIGHT
      K=1
 1    IF (K.EQ.NZ-1.OR.NZ.EQ.NZMAX) GOTO 3
      IF (DZ(K+1).GT.MAX(1.001*SECAUR*DZ(K),DZMIN)) THEN
**       SPLIT INTERVALL K+1 APPROPRIATELY
         Z(NZ+1)=Z(NZ)
         DO 2 I=NZ,K+3,-1
         Z(I)=Z(I-1)
 2       DZ(I)=DZ(I-1)
         Z(K+2)=Z(K+1)+DZ(K+1)*(2-SECAUR)
         DZ(K+2)=Z(K+3)-Z(K+2)
         DZ(K+1)=Z(K+2)-Z(K+1)
         NZ=NZ+1
***
      ELSE
         K=K+1
      ENDIF
**    CHECK NEXT INTERVALL
*     K=K+1
      GOTO 1

**    CHECK TO THE LEFT
 3    K=NZ-1
 4    IF (K.EQ.1.OR.NZ.EQ.NZMAX) GOTO 6
      IF (DZ(K-1).GT.MAX(1.001*SECAUR*DZ(K),DZMIN)) THEN
**       SPLIT INTERVALL K-1 APPROPRIATELY
         Z(NZ+1)=Z(NZ)
         DO 5 I=NZ,K+1,-1
         Z(I)=Z(I-1)
 5       DZ(I)=DZ(I-1)
         Z(K)=Z(K-1)+DZ(K-1)*(SECAUR-1)
         DZ(K)=Z(K+1)-Z(K)
         DZ(K-1)=Z(K)-Z(K-1)
         NZ=NZ+1
         K=K+1
      ELSE
         K=K-1
      ENDIF
      GOTO 4

 6    RETURN
      END
      SUBROUTINE GRIDF (KMIN,KMAX,KSTEP,SECAUR,NZ,MAXNPT,Z)
      INCLUDE 'TYPES.INC'

***   GRIDF - FILL GRID
*
*     W.MADER        1987        SIEMENS VIENNA
*
      DIMENSION Z(MAXNPT)

**    START OF EXECUTABLE CODE
*     ************************
*
      ZINC=(Z(KMAX)-Z(KMIN))/KSTEP

      NZINS=INT((Z(KMAX)-Z(KMAX-1))/ZINC+(2-SECAUR))-1
      IF(NZINS.GE.1) THEN
         CALL GRIDS (NZINS,KMAX,NZ,MAXNPT,Z)
         DO 1 I=KMAX+NZINS-1,KMAX,-1
1        Z(I)=Z(I+1)-ZINC
      ENDIF

      IF(KMIN.NE.KMAX-1) THEN
         NZINS=INT((Z(KMIN+1)-Z(KMIN))/ZINC+(2-SECAUR))-1
         IF(NZINS.GE.1) THEN
            CALL GRIDS (NZINS,KMIN+1,NZ,MAXNPT,Z)
            DO 2 I=KMIN+1,KMIN+NZINS,1
2           Z(I)=Z(I-1)+ZINC
         ENDIF
      ENDIF

      END
      SUBROUTINE GRIDS (NZINS,KMIN,NZ,MAXNPT,Z)
      INCLUDE 'TYPES.INC'

***   GRIDS - SHIFT GRID
*
*     W.MADER        1987        SIEMENS VIENNA
*
      DIMENSION Z(MAXNPT)

**    START OF EXECUTABLE CODE
*     ************************
*
      NZ=NZ+NZINS
      IF(NZ.GT.MAXNPT) RETURN
      DO 1 I=NZ,KMIN+NZINS,-1
1     Z(I)=Z(I-NZINS)

      END
      SUBROUTINE LIMITS (EPSMA,SMALL,BERMIN)
      INCLUDE 'TYPES.INC'

***   LIMITS - CALCULATE PRECISION LIMITS OF HARWARE
*
*     S.SELBERHERR   1979-1986
*

***   ON EXIT
*
*     ASSUME: B = BASE OF ARITHMETIC
*             T = NUMBER OF BASE B DIGITS
*             L = SMALLEST POSSIBLE EXPONENT
*
*     EPSMA  - ACCURACY OF 1.000...
*              EPSMA = B**(1-T)
*     SMALL  - SMALLEST POSITIVE AND "SAVE" REAL QUANTITY
*              SMALL = 100*B**(-L+T)
*              IT IS ASSUMED THAT 1/SMALL IS A VALID QUANTITY
*     BERMIN - MINIMUM VALUE FOR WHICH TO CALCULATE BERNOULLI FUNCTION
*              FOR ABS(X).LE.BERMIN : X/(EXP(X)-1) =
*               = 1-X/2+X**2/12-X**4/720+X**6/30420
*
*     IF THE CALCULATION OF THESE QUANTITIES CAUSES TROUBLES,
*     THEY SHOULD BE SET EXPLICITELY BY ASSIGNMENT STATEMENTS
*
      PARAMETER (EIGHT=8,FOUR=4,TWO=2,ONE=1,ZERO=0)
      PARAMETER (P1=ONE/2,P2=ONE/12,P3=ONE/720,P4=ONE/30240)

      X=ONE
 1    X=X/TWO
      CALL LIMITT (X1,ONE,X)
      IF (X1.GT.ONE) GOTO 1
      EPSMA=X
      X=X*EIGHT*EIGHT
 2    X=X/TWO
      CALL LIMITT (X1,ONE,-X)
      IF (X1.LT.ONE) GOTO 2
      EPSMA=TWO*MAX(EPSMA,X)

      X=ONE
 3    SMALL=X
      X=X/TWO
      CALL LIMITU (X1,EPSMA,X)
      IF (X1.NE.ZERO) GOTO 3

      XL=ZERO
      XU=TWO
      AERR=ONE
      RERR=ONE
 4    X=(XL+XU)/TWO
      IF (XU-XL.LE.FOUR*EPSMA*(XL+XU)) GOTO 5
      X1=ONE-X*(P1-X*(P2-X*X*(P3-X*X*P4)))
      X2=X/(EXP(X)-ONE)
      AERRO=AERR
      AERR=(X1-X2)
      RERRO=RERR
      RERR=TWO*AERR/(X1+X2)
      IF (RERR.LE.FOUR*EPSMA) THEN
         XL=X
      ELSEIF (AERR.LE.AERRO.AND.RERR.LE.RERRO) THEN
         XU=X
      ELSE
         XL=XL*(ONE+TWO*EPSMA)
      ENDIF
      GOTO 4

 5    BERMIN=X

      END
      SUBROUTINE LIMITT(X1,X2,X3)
      INCLUDE 'TYPES.INC'
      X1=X2+X3
      END
      SUBROUTINE LIMITU(X1,X2,X3)
      INCLUDE 'TYPES.INC'
      X1=X2*X3
      END
      SUBROUTINE NORMAL
      INCLUDE 'TYPES.INC'

***   NORMAL - INPUT PARAMETER SETUP
*
*     A.SCHUETZ      1980-1983
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
*     ESTIMATE HARDWARE LIMITS
      CALL LIMITS (EPSMA,SMALL,BERMIN)
      EXPLIM=-LOG(SMALL)
      ERFLIM=SQRT(EXPLIM)

**    SETUP CONSTANTS
*
*     SECTIO AUREA CONSTANT
      SECAUR=(1+SQRT(5*ONE))/2
**    COMPUTE ENERGY GAP PARAMETERS
      CALL GAPPAR (SIMTEM,EGAP,RMIEFF,RMAEFF,CSTAMI,CSTAMA,UT,CINTRI)
**    INTRINSIC CARRIER CONCENTRATION
      IF (CINTR.LE.0) CINTR=CINTRI
**    SCALING FACTOR FOR CONCENTRATIONS
*     MAXIMUM AND MINIMUM CARRIER CONCENTRATION
      CCMAX=2.2382/(28.0855*1.6604E-24)
      CNOR=SQRT(CCMAX)*SQRT(CINTR)
      CCMIN=MAX(SMALL,0.01*MIN(ONE,(CINTR/CCMAX)*CINTR)/CNOR)
      CCMAX=CCMAX/CNOR
      CINTR=CINTR/CNOR
**    LOG OF MAXIMUM AND MINIMUM CARRIER CONCENTRATION
      DPHMAX=LOG(CCMAX)
      DPHMIN=LOG(CCMIN)
**    DENSITY OF STATES
      CSTAMI=CSTAMI/CNOR
      CSTAMA=CSTAMA/CNOR
**    SCALING CONSTANT FOR LENGTHS
      XYNOR=SQRT((8.854187818E-14*UT/Q)*(ESEM/CNOR))
**    SCALING FACTOR FOR AMPS
      FLUNOR=Q*XYNOR**2*CNOR*XWGEO
**    INDICES FOR CARRIER TYPES
      INDXN=(3-ICHANL)/2
      INDXP=(3+ICHANL)/2

**    INPUT DATA NORMALIZATION
*
*     BIAS
      UB=UB                       /UT
      UD=UD                       /UT
      UG=UG                       /UT
      US=US                       /UT

*     DEVICE
      TINS=TINS                   /XYNOR
      XLGEO=XLGEO                 /XYNOR
      XWGEO=XWGEO                 /XYNOR
      YISUBS=YISUBS               /XYNOR
      YSEMI=YSEMI                 /XYNOR
      LBOTI=YSEMI.NE.0
      IF (SGAP.LE.0) THEN
         SGAP=10*TINS
      ELSE
         SGAP=SGAP                /XYNOR
      ENDIF
      IF (DGAP.LE.0) THEN
         DGAP=SGAP
      ELSE
         DGAP=DGAP                /XYNOR
      ENDIF

**    INTERFACE
      CISS=CISS                   /(CNOR*XYNOR)
      IF (EAMEV.GE.0) EAMEV=4*EXP(EAMEV/UT)
      IF (ECMED.GE.0) ECMED=2*EXP(ECMED/UT)
      RPS=RPS                     /(CNOR*XYNOR)
      SFSS=SFSS                   /(CNOR*XYNOR)
      SISS=SISS                   /XYNOR
      SNSS=SNSS                   /(CNOR*XYNOR)
      IF (XISS.GT.-1) XISS=XISS   /XYNOR

**    OPTION
      RBULK=RBULK                 *FLUNOR/UT
      RSOURC=RSOURC               *FLUNOR/UT
      RDRAIN=RDRAIN               *FLUNOR/UT
      IF (THCUR.EQ.0) THCUR=1E-7*XWGEO/XLGEO
      THCUR=THCUR                 /FLUNOR
      IF (UFB.LT.1E10) UFB=UFB    /UT*ICHANL
      H1=UTP(1)
      H2=UTP(2)
      UTP(INDXN)=H1
      UTP(INDXP)=H2

**    PROFILE
      SDTOX=SDTOX                 /XYNOR
      SUBS=SUBS                   /CNOR
      SURF=SURF                   /CNOR
      XOFF=XOFF                   /XYNOR
      YOFF=YOFF                   /XYNOR

**    RECOMBINATION
      T=SIMTEM/300
      IF (ALPHA(1).LT.0) ALPHA(1)=7.00E5*(0.57+0.43*T**2)
      IF (ALPHA(2).LT.0) ALPHA(2)=1.58E6*(0.58+0.42*T**2)
      H1=ALPHA(1)                 *XYNOR
      H2=ALPHA(2)                 *XYNOR
      ALPHA(INDXN)=H1
      ALPHA(INDXP)=H2
      IF (AUGER(1).LT.0) AUGER(1)=2.8E-31*T**0.14
      IF (AUGER(2).LT.0) AUGER(2)=9.9E-32*T**0.2
C     H1=AUGER(1)                 *CNOR**2
C     H2=AUGER(2)                 *CNOR**2
      H1=CNOR*AUGER(1)*CNOR
      H2=CNOR*AUGER(2)*CNOR
      AUGER(INDXN)=H1
      AUGER(INDXP)=H2
      IF (BETA(1).LT.0) BETA(1)=1.23E6*(0.625+0.375*T)
      IF (BETA(2).LT.0) BETA(2)=2.04E6*(0.67+0.33*T)
      H1=-BETA(1)                 *XYNOR/UT
      H2=-BETA(2)                 *XYNOR/UT
      BETA(INDXN)=H1
      BETA(INDXP)=H2
      H1=SRV(1)                   /XYNOR
      H2=SRV(2)                   /XYNOR
      SRV(INDXN)=H1
      SRV(INDXP)=H2
      H1=TAU(1)
      H2=TAU(2)
      TAU(INDXN)=H1
      TAU(INDXP)=H2
      H1=VRV(1)                   /XYNOR
      H2=VRV(2)                   /XYNOR
      VRV(INDXN)=H1
      VRV(INDXP)=H2

**    STEP
      DUB=DUB                     /UT
      DUD=DUD                     /UT
      DUG=DUG                     /UT

      IF (ICHANL.LT.0) THEN
**       CHANGE SIGN IF NECESSARY
         DUB=-DUB
         DUD=-DUD
         DUG=-DUG
         FLUNOR=-FLUNOR
         SURF=-SURF
         UB=-UB
         UD=-UD
         UG=-UG
         US=-US
**       DENSITY OF STATES
         H1=CSTAMI
         CSTAMI=CSTAMA
         CSTAMA=H1
**       EFFECTIVE MASSES
         H1=RMIEFF
         RMIEFF=RMAEFF
         RMAEFF=H1
      ENDIF

**    ADJUST BIASING TO ZERO SOURCE VOLTAGE
      IF (LBOTI) THEN
          UB=US
      ELSE
          UB=UB-US
      ENDIF
      UD=UD-US
      UG=UG-US

**    ACCOUNT FOR PROPER OFFSETS
      XLPRO1=XLGEO+MAX(ZERO,-XOFF)
      XLPRO2=XLPRO1+XOFF
      XOFF=ABS(XOFF)

**    VERY ROUGH ESTIMATES OF A FEW PARAMETERS
      YJD=MIN(XLGEO/4,1E-4/XYNOR)
      YJS=YJD

      WRITE (NUMREF,'(8(11X,1PE10.2,'' = '',A/))')
     1 EPSMA,'PRECISION OF 1.0',
     2 SMALL,'SMALLEST FLOATING POINT QUANTITY',
     3EXPLIM,'UPPER LIMIT FOR EXP(X)',
     4ERFLIM,'UPPER LIMIT FOR ERF(X)',
     5BERMIN,'LOWER LIMIT FOR BER(X)',
     6    UT,'SCALING FACTOR FOR POTENTIALS',
     7 XYNOR,'SCALING FACTOR FOR LENGTHS',
     8  CNOR,'SCALING FACTOR FOR CONCENTRATIONS'

      END
      SUBROUTINE RECOMB (H1,H2,H3,IKIND)
      INCLUDE 'TYPES.INC'

***   RECOMB - CALCULATE RECOMBINATION/GENERATION RATE
*              WITHOUT IMPACT IONIZATION
*
*     S.SELBERHERR   1986
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      DIMENSION H1(NXY),H2(NXY),H3(NXY)

**    START OF EXECUTABLE CODE
*     ************************
*
*     SHOCKLEY-READ-HALL AND AUGER RECOMBINATION
      H=CINTR**2
      TAU1=TAU(IKIND)
      TAU2=TAU(3-IKIND)
      AUGER1=AUGER(IKIND)
      AUGER2=AUGER(3-IKIND)
      DO 1 L=NX+1,NXYMX
      CC1=H3(L)
      CC2=H2(L)
      A1=CC1*CC2-H
      A2=1/(TAU2*(CC1+CINTR)+TAU1*(CC2+CINTR))
      H1(L)=A1*(A2+AUGER1*CC1+AUGER2*CC2)
 1    H2(L)=(CC2-TAU2*A1*A2)*A2+CC2*(AUGER2*CC2+AUGER1*(2*CC1-H/CC2))

**    SURFACE RECOMBINATION
      TAU1=1/SRV(IKIND)
      TAU2=1/SRV(3-IKIND)
      HDY=DY(NYOX)/2
      DO 2 L=NXS+1,NXD-1
      CC1=H3(L)
      CC2=H2(L)
      A1=HDY*(CC1*CC2-H)
      A2=1/(TAU2*(CC1+CINTR)+TAU1*(CC2+CINTR))
      H1(L)=H1(L)+A1*A2
 2    H2(L)=H2(L)+(HDY*(CC2-TAU2*A1*A2))*A2

      IF (LBOTI) THEN
**       INTERFACE RECOMBINATION
         TAU1=1/VRV(IKIND)
         TAU2=1/VRV(3-IKIND)
         HDY=DY(NYSEM-1)/2
         DO 3 L=NXYMX+1,NXY
         CC1=H3(L)
         CC2=H2(L)
         A1=HDY*(CC1*CC2-H)
         A2=1/(TAU2*(CC1+CINTR)+TAU1*(CC2+CINTR))
         H1(L)=H1(L)+A1*A2
 3       H2(L)=H2(L)+(HDY*(CC2-TAU2*A1*A2))*A2
      ENDIF

      END
      SUBROUTINE SUB02A
      INCLUDE 'TYPES.INC'

***   SUB02A - *INITIAL* PROCESSOR
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      INCLUDE 'CPADBLK.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
      CALL ZZZSEC (HCPU)
      HCPU=HCPU-CPUTIM
      WRITE (NUMREF,'(/1X,F9.3,1X,A/)') HCPU,'INITIAL PROCESSING'

**    NORMALIZE INPUT DATA
      CALL NORMAL
**    BUILD VERY INITIAL GRID
      CALL GRID
**    RESET GRID ADAPTIONS COUNTER
      ITGRID=0

**    LOOP TO ADAPT THE MESH TO THE DOPING PROFILE
*
 1    ITGRID=ITGRID+1
**    FIND MILESTONES IN MESH
      CALL FIMILE
**    SHOW GRID INFORMATION
      WRITE (NUMREF,
     1'(11X,A6,4(I3,A2),I3,A6,2(I3,A2),I3,A5,I3,A3,I3)')
     2'GRID (',NXS,' +',NXG1-NXS-1,' +',NXG2-NXG1+1,' +',
     3NXD-NXG2-1,' +',NX-NXD+1,' ) * (',
     4NYOX-1,' +',NY,' +',NYTOT-NYSEM,' ) = ',
     5NX,' *',NYTOT
**    PERFORM MEMORY HANDLING
      CALL MEMRY
**    COMPUTE DOPING PROFILE
      CALL CONCNT (PAD(JP(2)),PAD(JP(3)),
     1PAD(JP(1)),PAD(JP(1)+MAX(MAXDO2,NX)),
     2PAD(JP(4)),PAD(JP(4)+MAXDO2*MAXDO2),PAD(JP(4)+2*MAXDO2*MAXDO2))
      IF (IFAIL.NE.0) GOTO 2
**    COMPUTE WEIGHTFUNCTION FOR THE GIVEN MESH
      CALL CCNET (PAD(JP(5)),PAD(JP(2)),PAD(JP(3)))
      CALL BWFDOP (PAD(JP(5)),PAD(JP(4)),PAD(JP(4)+MAXNPT-1),
     1PAD(JP(6)))
**    CHECK THE MESH FOR APPROPRIATENESS
      CALL CHECK (PAD(JP(4)),PAD(JP(4)+MAXNPT-1),LGRDOK)
**    ITERATE ONCE MORE UNTIL GRID IS OK
      IF (.NOT.LGRDOK) GOTO 1

**    CLEAR SLOT POINTERS
 2    DO 3 I=1,MAXSLT
 3    IMEM(I)=JSLPRE
**    OPEN WORKING FILE
      CALL SLTOPN (NXYTOT)
**    SAVE DONOR AND ACCEPTOR CONCENTRATION
      CALL SLTPUT (PAD(JP(2)),ICDON,NXY)
      CALL SLTPUT (PAD(JP(3)),ICACC,NXY)

      END
      SUBROUTINE SUB02C
      INCLUDE 'TYPES.INC'

***   SUB02C - *ITERATION* PROCESSOR FOR MODEL=2-D
*
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      INCLUDE 'CPADBLK.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
      CALL ZZZSEC (HCPU)
      HCPU=HCPU-CPUTIM
      WRITE (NUMREF,'(/1X,F9.3,1X,A)') HCPU,'ITERATION FOR MODEL = 2-D'

**    NOW WE ACTIVATE BULK FULLY
      NT=NY
      NXT=NX*NT
      NXTMX=NXT-NX

      IMODEL=3
      ITCYC=0
      CALL SUB02I (.TRUE.,.FALSE.,1)

**    SAVE CURRENT OF MODEL=2-D
      FLUX2D=FLUXD(1)

      END
      SUBROUTINE SUB02D
      INCLUDE 'TYPES.INC'

***   SUB02D - *ITERATION* PROCESSOR FOR MODEL=AVL
*
*     S.SELBERHERR   1979-1986
*
      PARAMETER (MAXHOT=20)
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      INCLUDE 'CPADBLK.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
      CALL ZZZSEC (HCPU)
      HCPU=HCPU-CPUTIM
      WRITE (NUMREF,'(/1X,F9.3,1X,A)') HCPU,'ITERATION FOR MODEL = AVL'

      IMODEL=4
      ITHOT=0
      ITCYC=0

 1    CALL SUB02I (.FALSE.,.FALSE.,2)

      IF (ITHOT.EQ.0) THEN
         FLUXAV(1)=FLUXD(1)-FLUXD(2)
         FLUXAV(2)=FLUXB(1)-FLUXB(2)
      ENDIF
      IF (MODEL.EQ.4.OR.IFAIL.NE.0) GOTO 3

**    CHECK HOT ELECTRON CONVERGENCE
      IF (ITHOT.GE.MAXHOT) GOTO 2
      IF (PNORH.LE.ERRTC.AND.ITHOT.NE.0) GOTO 3
**    ONE MORE HOT CYCLE
      IMODEL=5
      IPM=0
      IPMPT=0
      ITHOT=ITHOT+1
      GOTO 1

 2    IFAIL=5
 3    RETURN
      END
