      SUBROUTINE BWF1D (PSI,CJ,BWFX,BWFY,HLP,LCHALL)
      INCLUDE 'TYPES.INC'

***   BWF1D - COMPUTE WEIGHT FUNCTION FOR MODEL=1-D
*
*     S.SELBERHERR   1987
*     J.FARICELLI    1987    DEC-HUDSON
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      DIMENSION PSI(NX,NYTOT),CJ(NX,NT),BWFX(MAXNPT-1),BWFY(MAXNPT-1),
     1HLP(NX,NYTOT)

**    START OF EXECUTABLE CODE
*     ************************
*
*     MINIMUM SPACING IS 5 ANGSTROM
      XYMIN=0.5E-7*SECAUR/XYNOR
      IF (LCHALL) THEN
**       COMPUTE WEIGHT FUNCTION OF X-MESH
         IS=MAX(NXJS-2,NXG1-2,NXS+1)
         IE=MIN(NXJD+2,NXG2+2,NXD-1)
         IF (LBOTI) THEN
            JE=NYSEM
         ELSE
            JE=NYOX+MIN(NYJS,NYJD)
         ENDIF
         DO 2 I=1,NX-1
         IF (DX(I).LE.XYMIN) THEN
            BWFX(I)=0
         ELSE
            H=0
            DO 1 J=1,NYTOT
 1          H=MAX(H,ABS(PSI(I,J)-PSI(I+1,J)))
            BWFX(I)=(UT/3)*MAX(4*BWFX(I),H)
         ENDIF
 2       CONTINUE
         DO 4 I=IS,IE-1
         IF (DX(I).GT.XYMIN) THEN
            H=0
            DO 3 J=NYOX,JE
 3          H=MAX(H,ABS(PSI(I,J)-PSI(I+1,J)))
            BWFX(I)=MAX(BWFX(I),(UT/1.5)*H)
         ENDIF
 4       CONTINUE

**       COMPUTE WEIGHT FUNCTION OF Y-MESH
         DO 6 J=1,NYTOT-1
         IF (DY(J).LE.XYMIN) THEN
            BWFY(J)=0
         ELSE
            H=0
            DO 5 I=1,NX
 5          H=MAX(H,ABS(PSI(I,J)-PSI(I,J+1)))
            BWFY(J)=(UT/3)*MAX(4*BWFY(J),H)
         ENDIF
 6       CONTINUE
         DO 8 J=NYOX,JE-1
         IF (DY(J).GT.XYMIN) THEN
            H=0
            DO 7 I=IS,IE
 7          H=MAX(H,ABS(PSI(I,J)-PSI(I,J+1)))
            BWFY(J)=MAX(BWFY(J),(UT/1.5)*H)
         ENDIF
 8       CONTINUE

**       NOW CHECK CURRENT DENSITIES
         FLH=(1/0.05)/(MAX(ABS(FLUXD(1)),1E-10/ABS(FLUNOR)))
         CALL SLTGET (CJ,IJD(3),NXTMX)
         DO 9 J=1,NT-2
         DO 9 I=IS,IE
 9       HLP(I,J)=ABS(FWX(I)*DFWY(J)*CJ(I,J))
         DO 11 I=IS,IE-1
         IF (DX(I).GT.XYMIN) THEN
            H=0
            DO 10 J=1,NT-2
 10         H=MAX(H,HLP(I,J),HLP(I+1,J))
            BWFX(I)=MAX(BWFX(I),FLH*H)
         ENDIF
 11      CONTINUE
         CALL SLTGET (CJ,IJD(1),NXT-1)
         DO 12 J=1,NT-1
         DO 12 I=IS,IE-1
 12      HLP(I,J)=ABS(DFWX(I)*FWY(J)*CJ(I,J))
         DO 14 J=1,NT-2
         IF (DY(J+NYOX-1).GT.XYMIN) THEN
            H=0
            DO 13 I=IS,IE-1
 13         H=MAX(H,HLP(I,J),HLP(I,J+1))
            BWFY(J+NYOX-1)=MAX(BWFY(J+NYOX-1),FLH*H)
         ENDIF
 14      CONTINUE
         IF (MODEL.GE.4) THEN
**          COMPUTE ELECTRIC FIELD IN SEMICONDUCTOR REGION
            CALL DIFFX (HLP,PSI(1,NYOX),DX,NX,NT)
**          ESTIMATE AVALANCE GENERATION AT MID POINTS BETWEEN MESH
*           WE ONLY HAVE MINORITY CURRENTS, SO WE CAN ONLY
*           ESTIMATE MINORITY CONTRIBUTION.
*           ESM IS MINIMUM FILED FOR GENERATION
            ESM=-BETA(1)*0.05
            DO 15 J=1,NT
            DO 15 I=1,NX-1
*           EPAR IS EX IN DIRECTION OF JX
            EPAR=-SIGN(ONE,CJ(I,J))*HLP(I,J)
            IF (EPAR.GE.ESM) THEN
               HLP(I,J)=ABS(CJ(I,J))*ALPHA(1)*EXP(BETA(1)/EPAR)
            ELSE
               HLP(I,J)=0
            ENDIF
 15         CONTINUE

**          NOW MODIFY BWFX AND BWFY USING GENERATION RATE
*           GREF IS BASE GENERATION RATE. ANYTHING BELOW THIS WILL NOT
*           TRIGGER REFINEMENT.
            GREF=1E22/CNOR
*           MINIMUM GRID SPACING FOR AVALANCE REGION IS 50 ANGSTROMS
            AXYMIN=60E-8/XYNOR*SECAUR

**          SINCE GENERATION RATE VARIES MANY ORDERS OF MAGNITUDE
*           USE LOG WEIGHTING SCHEMA AS IN BWFDOP.
            DO 16 J=1,NT
            DO 16 I=1,NX-1
 16         HLP(I,J)=LOG10(MAX(ONE+1E-3,HLP(I,J)/GREF))

**          USE AVERAGE OF GENERATION RATE OVER TWO POINTS TO
*           TRIGGER REFINEMENT
*           X MESH
            DO 18 I=1,NX-1
            IF (DX(I).GT.AXYMIN) THEN
               H=0
               DO 17 J=1,NT
 17            H=MAX(H,HLP(I,J))
               H=H*(DX(I)-AXYMIN)/DX(I)
               BWFX(I)=MAX(BWFX(I),H)
            ENDIF
 18         CONTINUE
*           Y MESH (SEMICONDUCTOR REGION ONLY )
            DO 20 J=1,NT-1
            IF (DY(J+NYOX-1).GT.AXYMIN) THEN
               H=0
               DO 19 I=1,NX-1
 19            H=MAX(H,HLP(I,J)+HLP(I+1,J)+HLP(I,J+1)+HLP(I+1,J+1))
               H=H/4*(DY(J+NYOX-1)-AXYMIN)/DY(J+NYOX-1)
               BWFY(J+NYOX-1)=MAX(BWFY(J+NYOX-1),H)
            ENDIF
 20         CONTINUE
         ENDIF
      ELSE
         DO 21 I=1,NX-1
         IF (DX(I).LE.XYMIN) THEN
            BWFX(I)=0
         ELSE
            H=ABS(PSI(I,NYOX)-PSI(I+1,NYOX))
            BWFX(I)=(UT/2.5)*H
         ENDIF
 21      CONTINUE
         DO 22 J=1,NYTOT-1
 22      BWFY(J)=0
      ENDIF
**    ASSURE THAT OXIDE IS WEIGHTED PROPERLY
      BWFY(NYOX-1)=BWFY(NYOX)
      IF (LBOTI) BWFY(NYSEM)=BWFY(NYSEM-1)

      END
      SUBROUTINE INIT (PSI,CMIN,CMAJ,CNET)
      INCLUDE 'TYPES.INC'

***   INIT - ESTIMATE INITIAL SOLUTION
*
*     A.SCHUETZ      1980-1983
*     S.SELBERHERR   1979-1986
*
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      DIMENSION PSI(NX,NYTOT),CMIN(NXY),CMAJ(NXY),CNET(NXY)

**    CHANNEL CHARGE
      QCH(U)=DIM(UGS,U+CDEBE*SQRT(2*DIM(U,UB-UF+1)))
**    QUASIFERMILEVEL
      PHI(Z)=A5*ABS(Z*(A1+Z*(A2+Z*A3)))**A4

      PYT(REAL1,REAL2)=SQRT(REAL1**2+REAL2**2)

**    START OF EXECUTABLE CODE
*     ************************
*
*     COMPUTE NET DOPING CONCENTRATION
      CALL SLTGET (CNET,ICDON,NXY)
      CALL SLTGET (CMAJ,ICACC,NXY)
      CALL CCNET (CNET,CNET,CMAJ)
      CALL FIXJ (CNET)
      IF (IFAIL.NE.0) RETURN

      XLEFF=MAX(0.1*XLGEO,XLGEO-XJS-XJD)
      XHELP=MAX(0.2*XLGEO,XLGEO-(XJS-XJD)/2)

**    FERMI VOLTAGE
      R2CI=2*CINTR
      UL2CI=LOG(R2CI)
      UF=LOG(SUBS+PYT(SUBS,R2CI))-UL2CI

**    FLAT-BAND VOLTAGE
      IF (UFB.GE.1E10) UFB=GATE/UT*ICHANL-UF

**    EFFECTIVE GATE BIAS
      UGS=UG-UFB-UF

**    EFFECTIVE SUBSTRATE DOPING
      SUBSCH=0
      H=0
      DO 1 J=1,NY
      IF (Y(J+NYOX-1).GT.4E-7/XYNOR) GOTO 2
      DO 1 I=MIN(NXJD-2,NXJS+2),MAX(NXJS+2,NXJD-2)
      L=(J-1)*NX+I
      SUBSCH=SUBSCH+DXM(I)*DYM(J+NYOX-1)*ABS(CNET(L))
 1    H=H+DXM(I)*DYM(J+NYOX-1)
 2    SUBSCH=MAX(SUBSCH/H,SUBS/10)

**    EXTRINSIC DEBEYE CAPACITANCE (/INSULATOR-CAPACITANCE)
      CDEBE=TINS*XYNOR*SQRT((Q/(8.854187818E-14*UT))*ESEM*SUBSCH)*
     1SQRT(CNOR)/EINS

**    LONG-CHANNEL THRESHOLD VOLTAGE
      UT0=UF+CDEBE*SQRT(2*DIM(2*UF,UB))
      IF (ABS(UG).GE.1E10) THEN
         UGS=UT0
         UG=UGS+UFB+UF
      ENDIF

**    SET BOUNDARY CONDITIONS
      CALL BNDVAL (PSI,CMIN,CMAJ,CNET,.TRUE.)

**    PRESET FROM CONTACTS TO THE INTERIOUR
      DO 3 I=NXS+1,NXJS-1
 3    PSI(I,NYOX)=LOG(CNET(I)+PYT(CNET(I),R2CI))-UL2CI
      DO 4 I=NXJD+1,NXD-1
 4    PSI(I,NYOX)=LOG(CNET(I)+PYT(CNET(I),R2CI))-UL2CI+UD

**    BULK BOUNDARAY FOR THE SOI CASE
      IF (LBOTI) THEN
         DO 5 I=1,NX
 5       PSI(I,NYSEM)=LOG(CNET(I+NXYMX)+PYT(CNET(I+NXYMX),R2CI))-UL2CI
      ENDIF

**    CONTACT POTENTIALS
      PSIS=PSI(NXS,NYOX)
      PSID=PSI(NXD,NYOX)

**    FERMILEVEL IN THE CHANNEL
      UFC=LOG(SUBSCH+PYT(SUBSCH,R2CI))-UL2CI
      QCH0=QCH(UF)

      IF (UGS+UFC-UB-1.GE.0) THEN
**       IN WEAK INVERSION
         PSIC=UGS+CDEBE*(CDEBE-SQRT(CDEBE**2+2*(UGS+UFC-UB-1)))
         LSTRNG=.FALSE.
         IF (PSIC.GT.UFC+1) THEN
**          IN STRONG INVERSION
            PSIC=UFC+1
            LSTRNG=.TRUE.
         ENDIF
      ELSE
**       IN ACCUMULATION
         PSIC=-UFC
         LSTRNG=.FALSE.
      ENDIF

**    DEPLETION WIDTHS
      IF (PSIC.LT.PSIS) THEN
         XS=SQRT((PSIS-PSIC)/SUBSCH)
      ELSE
         XS=0.01*XHELP
      ENDIF
      IF (PSIC.LT.PSID) THEN
         XD=SQRT((PSID-PSIC)/SUBSCH)
      ELSE
         XD=0.01*XHELP
      ENDIF
      H=(XS+XD)/XLEFF
      IF (H.GT.1) THEN
         XS=XS/H
         XD=XD/H
      ENDIF

      IF (LSTRNG) THEN
**       STRONG INVERSION
         XJSH=XJS/2
         A1=QCH0/(QCH0+CDEBE*SQRT(UFC))
         A2=1-A1
         A3=-A2/1.5
         A4=1.5/(A1+0.5)
         A5=UD/ABS(A1+A2+A3)**A4
         DO 6 I=NXJS,NXJD
 6       PSI(I,NYOX)=UFC+PHI(MAX(ZERO,MIN((X(I)-XJSH)/XHELP,ONE)))
      ELSE
**       WEAK INVERSION
         A1=1.25*(PSI(NXJS-1,NYOX)-PSIC)/XS**2
         XH=X(NXJS-1)
         DO 7 II=NXJS,NXJD
         IF (X(II).GT.XH+0.2*XS) GOTO 8
 7       PSI(II,NYOX)=PSI(NXJS-1,NYOX)-4.*A1*(X(II)-XH)**2
 8       XH=XH+XS
         DO 9 I=II,NXJD
 9       PSI(I,NYOX)=PSIC+A1*(DIM(XH,X(I)))**2
         A1=(PSI(NXJD+1,NYOX)-PSIC)/(0.91*XD**2)
         XH=X(NXJD+1)
**       PINCH OFF REGION
         DO 10 II=NXJD,NXJS,-1
         IF (X(II).LT.XH-0.1*XD) GOTO 11
 10      PSI(II,NYOX)=PSI(NXJD+1,NYOX)-10*A1*(X(II)-XH)**2
 11      XH=XH-XD
         DO 12 I=II,NXJS,-1
         IF (X(I).LT.XH) GOTO 13
 12      PSI(I,NYOX)=PSI(I,NYOX)+A1*(XH-X(I))**2
      ENDIF

**    SOURCE - BULK
 13   DO 16 I=1,NXJS
      DO 14 J=NYOX+1,NYSEM-1
      L=(J-NYOX-1)*NX+I
      IF (CNET(L).LT.SUBS) GOTO 15
 14   PSI(I,J)=LOG(CNET(L)+PYT(CNET(L),R2CI))-UL2CI
 15   YH=SQRT(DIM(PSI(I,J-1),PSI(I,NYSEM))/SUBS)+Y(J-1)
      DO 16 JJ=J,NYSEM-1
 16   PSI(I,JJ)=PSI(I,NYSEM)+SUBS*(DIM(YH,Y(JJ)))**2

**    CHANNEL - BULK
      IF (LSTRNG) THEN
         H=-2*SQRT(2*SUBSCH)
         DO 17 I=NXJS,NXJD
         YH=SQRT(DIM(PSI(I,NYOX)-UFC,PSI(I,NYSEM))/SUBSCH)
         DO 17 J=NYOX+1,NYSEM-1
 17      PSI(I,J)=PSI(I,NYSEM)+SUBSCH*(DIM(YH,Y(J)))**2+
     1   UFC*EXP(MAX(Y(J)*H,-EXPLIM))
      ELSE
         DO 18 I=NXJS,NXJD
         YH=SQRT(DIM(PSI(I,NYOX),PSI(I,NYSEM))/SUBSCH)
         DO 18 J=NYOX+1,NYSEM-1
 18      PSI(I,J)=PSI(I,NYSEM)+SUBSCH*(DIM(YH,Y(J)))**2
      ENDIF

**    DRAIN - BULK
      DO 21 I=NXJD,NX
      DO 19 J=NYOX+1,NYSEM-1
      L=(J-NYOX-1)*NX+I
      IF (CNET(L).LT.SUBS) GOTO 20
 19   PSI(I,J)=LOG(CNET(L)+PYT(CNET(L),R2CI))-UL2CI+UD
 20   YH=SQRT(DIM(PSI(I,J-1),PSI(I,NYSEM))/SUBS)+Y(J-1)
      DO 21 JJ=J,NYSEM-1
 21   PSI(I,JJ)=PSI(I,NYSEM)+SUBS*(DIM(YH,Y(JJ)))**2

**    SEW SOURCE WITH CHANNEL AND CHANNEL WITH DRAIN
      HS=DX(NXJS-1)/(DX(NXJS-1)+DX(NXJS))
      HD=DX(NXJD-1)/(DX(NXJD-1)+DX(NXJD))
      DO 22 J=NYOX+1,NYSEM-1
      PSI(NXJS,J)=PSI(NXJS-1,J)*(1-HS)+PSI(NXJS+1,J)*HS
 22   PSI(NXJD,J)=PSI(NXJD-1,J)*(1-HD)+PSI(NXJD+1,J)*HD

**    TOP INSULATOR POTENTIAL
      DO 24 J=2,NYG
      H=(PSI(NXG1,J)-PSI(NXS,J))/(X(NXG1)-X(NXS))
      DO 23 I=NXS+1,NXG1-1
 23   PSI(I,J)=PSI(NXS,J)+H*(X(I)-X(NXS))
      H=(PSI(NXD,J)-PSI(NXG2,J))/(X(NXD)-X(NXG2))
      DO 24 I=NXG2+1,NXD-1
 24   PSI(I,J)=PSI(NXG2,J)+H*(X(I)-X(NXG2))
      DO 25 J=NYG+1,NYOX-1
      H=Y(J)/TINS
      DO 25 L=NXS+1,NXD-1
 25   PSI(L,J)=PSI(L,NYOX)+H*(PSI(L,NYOX)-UGS)

**    BOTOM INSULATOR POTENTIAL
      DO 26 J=NYSEM+1,NYTOT
      DO 26 L=1,NX
 26   PSI(L,J)=PSI(L,NYSEM)

**    SAVE INITIAL GUESS
      CALL SLTPUT (PSI,IPSI,NXYTOT)

**    INITIALIZE CARRIER CONCENTRATIONS
      CALL INITC (PSI(1,NYOX),CMIN,CMAJ,CNET,3)

      END
      SUBROUTINE INITC (PSI,CMIN,CMAJ,CNET,NKIND)
      INCLUDE 'TYPES.INC'

***   INITC - INITIALIZE MINORITY CARRIER CONCENTRATION
*
*     S.SELBERHERR   1986
*
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      DIMENSION PSI(NXY),CMIN(NXY),CMAJ(NXY),CNET(NXY),IS(2),IE(2)

      PYT(REAL1,REAL2)=SQRT(REAL1**2+REAL2**2)

***    START OF EXECUTABLE CODE
*     ************************
*
      IS(1)=NXS+1
      IE(1)=NXD-1
      IS(2)=NX+1
      IE(2)=NXYB

**    INITIALIZE MINORITY CARRIER CONCENTRATION
      IF (MOD(NKIND,2).NE.0) THEN
         H1=2*CINTR
         H2=H1*CINTR
CVD$ NODEPCHK       ALLIANT DIRECTIVE
         DO 1 I=1,2
         DO 1 L=IS(I),IE(I)
         H=PYT(CNET(L),H1)
         IF (CNET(L).LT.0) THEN
            CMIN(L)=MAX(H2/(H-CNET(L)),CCMIN)
         ELSE
            CMIN(L)=(H+CNET(L))/2
         ENDIF
 1       CONTINUE
         CALL SLTPUT (CMIN,ICC(1),NXY)
      ENDIF

**    INITIALIZE MAJORITY CARRIER CONCENTRATION
      IF (NKIND.GT.1) THEN
         H=LOG(CINTR)+UB
CVD$ NODEPCHK       ALLIANT DIRECTIVE
         DO 2 I=1,2
         DO 2 L=IS(I),IE(I)
 2       CMAJ(L)=EXP(MIN(MAX(DPHMIN,H-PSI(L)),DPHMAX))
         CALL SLTPUT (CMAJ,ICC(2),NXY)
      ENDIF

      END
      SUBROUTINE SUB02B
      INCLUDE 'TYPES.INC'

***   SUB02B - *ITERATION* PROCESSOR FOR MODEL=1-D
*
*     S.SELBERHERR   1979-1986
*
      PARAMETER (MAXGRI=20)
      INCLUDE 'CMSYSX.INC'
      INCLUDE 'CMDATX.INC'
      INCLUDE 'CMOS02X.INC'
      INCLUDE 'CMEMX.INC'
      INCLUDE 'CPADBLK.INC'

**    START OF EXECUTABLE CODE
*     ************************
*
      CALL ZZZSEC (HCPU)
      HCPU=HCPU-CPUTIM
      WRITE (NUMREF,'(/1X,F9.3,1X,A)') HCPU,'ITERATION FOR MODEL = 1-D'
      IMODEL=1
      ITGRID=0
      NCHECK=0
      LUTHOK=.TRUE.
      OLDI=0
      PNORH=2*ERRTC
      LCHALL=.FALSE.

**    UPDATE BOUNDARIES AND INITIAL SOLUTION
 1    CALL INIT (PAD(JP(1)),PAD(JP(2)),PAD(JP(3)),PAD(JP(4)))
      IF (IFAIL.NE.0) RETURN

 2    IF ((ITGRD.EQ.0.OR..NOT.LGRDFZ).AND.ITGRID.LE.MAXGRI) THEN
**       SAVE TEMPERATURE BWF
         IF (LCHALL) THEN
            DO 7801 L=1,NX-1
 7801       PAD(L+JP(5)-1)=PAD(L+JP(10)-1)
            DO 7802 L=1,NYOX-1
 7802       PAD(L+JP(6)-1)=0
            DO 7803 L=1,NT-1
 7803       PAD(L+JP(6)+NYOX-2)=PAD(L+JP(10)+NX-2)
            DO 7804 L=NYOX+NT-1,NYTOT-1
 7804       PAD(L+JP(6)-1)=0
         ENDIF
**       EVALUATE WEIGHT FUNCTION FOR MESH
         CALL BWF1D (PAD(JP(1)),PAD(JP(3)),PAD(JP(5)),PAD(JP(6)),
     1   PAD(JP(7)),LCHALL)
**       SAVE PRESENT MESH
         NXOLD=NX
         DO 7701 L=1,NXOLD
 7701    PAD(L+JP(10)-1)=X(L)
         NYOLD=NY
         NYOXLD=NYOX
         NYTOLD=NYTOT
         DO 7702 L=1,NYTOLD
 7702    PAD(L+JP(10)-1+NXOLD)=Y(L)
**       CHECK THE MESH FOR APPROPRIATENESS
         CALL CHECK (PAD(JP(5)),PAD(JP(6)),LGRDOK)
         NCHECK=NCHECK+1

         IF (.NOT.LGRDOK) THEN
            ITGRID=ITGRID+1
**          UPDATE MILESTONES IN MESH
            CALL FIMILE
**          SHOW GRID INFORMATION
            WRITE (NUMREF,
     1      '(11X,A6,4(I3,A2),I3,A6,2(I3,A2),I3,A5,I3,A3,I3)')
     2      'GRID (',NXS,' +',NXG1-NXS-1,' +',NXG2-NXG1+1,' +',
     3      NXD-NXG2-1,' +',NX-NXD+1,' ) * (',
     4      NYOX-1,' +',NY,' +',NYTOT-NYSEM,' ) = ',
     5      NX,' *',NYTOT
**          INTERPOLATE POTENTIAL DISTRIBUTION
            CALL BICUB (PAD(JP(10)),PAD(JP(10)+NXOLD),PAD(JP(1)),
     1      PAD(JP(9)),PAD(JP(8)),X,Y,PAD(JP(2)),
     2      NXOLD,NYTOLD,NX,NX,NYTOT,ZERO,0)
**          INTERPOLATE MINORITY CARRIER CONCENTRATION
            CALL SLTGET (PAD(JP(1)),ICC(1),NXOLD*NYOLD)
            CALL BICUB (PAD(JP(10)),PAD(JP(10)+NXOLD+NYOXLD-1),
     1      PAD(JP(1)),PAD(JP(9)),PAD(JP(8)),X,Y(NYOX),
     2      PAD(JP(2)+NXYTOT),NXOLD,NYOLD,NX,NX,NY,ZERO,1)
**          CLOSE THE WORKING FILE
            CALL SLTCLS
**          RESET WORKING FILE POINTERS
            DO 3 I=1,MAXSLT
 3          IMEM(I)=JSLPRE
**          OPEN NEW WORKING FILE
            CALL SLTOPN (NXYTOT)
**          SAVE INTERPOLATED POTENTIAL AND MINORITY CARRIER CONCENTRATION
            CALL SLTPUT (PAD(JP(2)),IPSI,NXYTOT)
            CALL SLTPUT (PAD(JP(2)+NXYTOT),ICC(1),NXY)
**          UPDATE MEMORY REQUIREMENTS
            CALL MEMRY
**          CALCULATE NEW DOPING CONCENTRATION
            CALL CONCNT (PAD(JP(2)),PAD(JP(3)),
     1      PAD(JP(1)),PAD(JP(1)+MAX(MAXDO2,NX)),
     2      PAD(JP(4)),PAD(JP(4)+MAXDO2*MAXDO2),
     3      PAD(JP(4)+2*MAXDO2*MAXDO2))
            CALL SLTPUT (PAD(JP(2)),ICDON,NXY)
            CALL SLTPUT (PAD(JP(3)),ICACC,NXY)
            IF (IFAIL.NE.0) RETURN
**          UPDATE JUNCTION DEPTHS INDICES
            CALL CCNET (PAD(JP(2)),PAD(JP(2)),PAD(JP(3)))
            CALL FIXJ (PAD(JP(2)))
            IF (IFAIL.NE.0) RETURN
**          RELOAD POTENTIAL AND MINORITY CARRIER CONCENTRATION
            CALL SLTGET (PAD(JP(1)),IPSI,NXYTOT)
            CALL SLTGET (PAD(JP(3)),ICC(1),NXY)
**          GUARANTEE BOUNDARY CONDITIONS
            CALL BNDVAL (PAD(JP(1)),PAD(JP(3)),PAD(JP(4)),PAD(JP(2)),
     1      .TRUE.)
**          INITIALIZE MAJORITY CARRIER CONCENTRATION
            CALL INITC (PAD(JP(1)+NXYOX),PAD(JP(3)),PAD(JP(4)),
     1      PAD(JP(2)),2)
         ENDIF
      ELSE
         LGRDOK=.TRUE.
      ENDIF

      IF (.NOT.LGRDOK.OR.NCHECK.EQ.1.OR..NOT.LUTHOK.OR.LGRDFZ) THEN
**       SOLVE WITH SIMPLIFIED CONTINUITY EQUATION
         ITCYC=0
         CALL SUB02I (.FALSE.,.TRUE.,1)
**       NOW ALLOW FULL MESH CHECK
         LCHALL=.TRUE.
         IF (IFAIL.EQ.2) GOTO 4
      ENDIF

      IF (ITGRID.LT.MAXGRI.AND.(.NOT.LGRDFZ.OR.ITGRD.EQ.0).AND.
     1IFAIL.NE.2.AND.(.NOT.LGRDOK.OR.NCHECK.LT.2)) GOTO 2
      IF (MODEL.NE.2) GOTO 4
      IMODEL=2
      ITGRID=MAXGRI+1
      CALL THRES (OLDI,LUTHOK)
      IF (LUTHOK) GOTO 4
      CALL SLTGET (PAD(JP(2)),ICC(1),NXY)
      CALL SLTGET (PAD(JP(3)),ICC(2),NXY)
      CALL SLTGET (PAD(JP(4)),ICDON,NXY)
      CALL SLTGET (PAD(JP(5)),ICACC,NXY)
      CALL CCNET (PAD(JP(4)),PAD(JP(4)),PAD(JP(5)))
      CALL BNDVAL (PAD(JP(1)),PAD(JP(2)),PAD(JP(3)),PAD(JP(4)),.FALSE.)
      GOTO 2

**    REMEMBER THAT WE HAVE MODIFIED THE GRID
 4    ITGRD=ITGRD+1
**    SAVE CURRENT OF MODEL=1-D
      FLUX1D=FLUXD(1)

      END
